@@ -9,15 +9,14 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/conf.h>
+#include <sys/Thread.h>
 #include <lib/lib.h>
 #include <kern/proc.h>
-#include <kern/Thread.h>
 #include <kern/term.h>
-#include <kern/errno.h>
 #include <kern/vm.h>
 #include <kern/devfs.h>
 #include <kern/device.h>
-#include <kern/Wait.h>
+#include <kern/TaskWait.h>
 #include <dev/console/display.h>
 #include <dev/console/console.h>
 
@@ -82,9 +81,8 @@ STATIC void putNumScreen(
 	int i;
 
 	for (i = 0; i < size; ++i) {
-		putDisplay(getCursorPos() + i, str[i], ATTR_NORMAL);
+		putDisplay(str[i], ATTR_NORMAL);
 	}
-	updateCursor(getCursorPos() + size);
 }
 
 /*
@@ -93,12 +91,8 @@ STATIC void putNumScreen(
 STATIC void deleteBackScreen(
 	const size_t size)		// 
 {
-	int i;
-	
-	for (i = 0; i < size; ++i) {
-		putDisplay(getCursorPos() - i, '\0', ATTR_NORMAL);
-	}
-	updateCursor(getCursorPos() - size);
+	deleteDisplay(size);
+	leftDisplay(size);
 }
 
 /*
@@ -107,51 +101,7 @@ STATIC void deleteBackScreen(
 STATIC void deleteForwardScreen(
 	const size_t size)		// 
 {
-	int i;
-	
-	for (i = 0; i < size; ++i) {
-		putDisplay(getCursorPos() + i, '\0', ATTR_NORMAL);
-	}
-}
-
-/*
- * ̤õ
- */
-STATIC void deleteScreen()
-{
-	int pos = getScreenTop();
-	int i;
-	
-	for (i = 0; i < SCREEN_COLUMNS * SCREEN_LINES; ++i) {
-		putDisplay(pos + i, '\0', ATTR_NORMAL);
-	}
-	updateCursor(getCursorPos() + (SCREEN_COLUMNS * SCREEN_LINES));
-}
-
-/*
- * 򺸤ذư
- */
-STATIC void leftCursor(
-	int len)		// ư
-{
-	updateCursor(getCursorPos() - len);
-}
-
-/*
- * 򱦤ذư
- */
-STATIC void rightCursor(
-	int len)		// ư
-{
-	updateCursor(getCursorPos() + len);
-}
-
-/*
- * Ԥ
- */
-STATIC void newLine()
-{
-	updateCursor(ROUNDUP(getCursorPos() + 1, SCREEN_COLUMNS));
+	deleteDisplay(size);
 }
 
 /*
@@ -159,7 +109,7 @@ STATIC void newLine()
  */
 STATIC void carriageRetun()
 {
-	updateCursor(ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS));
+	leftDisplay(ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS));
 }
 
 /*
@@ -167,12 +117,12 @@ STATIC void carriageRetun()
  */
 STATIC void tab()
 {
-	int columns = ROUNDUP(getCursorPos() % SCREEN_COLUMNS + 1, TAB_SIZE);
+	int columns = ROUNDUP(getCurrentDisplay() % SCREEN_COLUMNS + 1, TAB_SIZE);
 
 	if (SCREEN_COLUMNS < columns) {
 		columns = SCREEN_COLUMNS;
 	}
-	updateCursor(columns + ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS));
+	rightDisplay(columns + ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS));
 }
 
 /*
@@ -182,13 +132,13 @@ STATIC void tab()
 STATIC void leftCursorTerm(int n)
 {
 	if (lbuf.crt < lbuf.pastCrt)
-		leftCursor(n);
+		leftDisplay(n);
 }
 
 STATIC void rightCursorTerm(int n)
 {
 	if (lbuf.pastCrt < lbuf.crt)
-		rightCursor(n);
+		rightDisplay(n);
 }
 
 STATIC void inputStrTerm(void)
@@ -199,7 +149,7 @@ STATIC void inputStrTerm(void)
 STATIC void eraseLineTerm(void)
 {
 	putNumScreen(lbuf.buf + lbuf.lineTop, lbuf.pastLast - lbuf.lineTop);
-	leftCursor(lbuf.pastCrt - lbuf.lineTop);
+	leftDisplay(lbuf.pastCrt - lbuf.lineTop);
 }
 
 STATIC void deleteTerm(void)
@@ -210,7 +160,7 @@ STATIC void deleteTerm(void)
 STATIC void backSpaceTerm(void)
 {
 	putNumScreen(lbuf.buf + lbuf.crt, lbuf.pastLast - lbuf.crt);
-	leftCursor(lbuf.pastCrt - lbuf.crt);
+	leftDisplay(lbuf.pastCrt - lbuf.crt);
 }
 
 static TERM_CTL termCtl = {
@@ -220,7 +170,7 @@ static TERM_CTL termCtl = {
 	leftCursorTerm,
 	rightCursorTerm,
 	inputStrTerm,
-	newLine,
+	newLineDisplay,
 	carriageRetun,
 	eraseLineTerm,
 	deleteTerm,
@@ -234,74 +184,81 @@ static TERM_CTL termCtl = {
 
 STATIC void setCursolInside(int x, int y)
 {
-	int position = SCREEN_COLUMNS * (y - 1) + (x - 1);
+/*	int position = SCREEN_COLUMNS * (y - 1) + (x - 1);
 
 	if (SCREEN_LINES * SCREEN_COLUMNS <= position) {
 		position = SCREEN_COLUMNS * SCREEN_LINES - 1;
 	}
 	updateCursor(getScreenTop() + position);
+*/
 }
 
 STATIC void leftCursolInside(int x)
 {
-	int position = getCursorPos() - x;
+/*	int position = getCurrentDisplay() - x;
 
 	if (position < getScreenTop()) {
 		position = getScreenTop();
 	}
 	updateCursor(position);
+*/
 }
 
 STATIC void rightCursolInside(int x)
 {
-	int position = getCursorPos() + x;
+/*	int position = getCurrentDisplay() + x;
 
 	if (getScreenTop() + SCREEN_COLUMNS * SCREEN_LINES <= position) {
 		position = getScreenTop() + SCREEN_COLUMNS * SCREEN_LINES - 1;
 	}
 	updateCursor(position);
+*/
 }
 
 STATIC void leftCursolLine(int x)
 {
-	int position = getCursorPos() - x;
+/*	int position = getCurrentDisplay() - x;
 
-	if (position < ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS)) {
-		position = ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS);
+	if (position < ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS)) {
+		position = ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS);
 	}
 	updateCursor(position);
+*/
 }
 
 STATIC void rightCursolLine(int x)
 {
-	int position = getCursorPos() + x;
+/*	int position = getCurrentDisplay() + x;
 
-	if (ROUNDUP(getCursorPos(), SCREEN_COLUMNS) <= position) {
-		position = ROUNDUP(getCursorPos(), SCREEN_COLUMNS) - 1;
+	if (ROUNDUP(getCurrentDisplay(), SCREEN_COLUMNS) <= position) {
+		position = ROUNDUP(getCurrentDisplay(), SCREEN_COLUMNS) - 1;
 	}
 	updateCursor(position);
+*/
 }
 
 STATIC void copyUpLine(int y)
 {
-	int src = ROUNDUP(getCursorPos(), SCREEN_COLUMNS);
+/*	int src = ROUNDUP(getCurrentDisplay(), SCREEN_COLUMNS);
 	int dst = src - y * SCREEN_COLUMNS;
 	
 	if (dst  < getScreenTop()) {
 		dst = getScreenTop();
 	}
 	copyDisplay(dst, src, getScreenTop() + SCREEN_LINES * SCREEN_COLUMNS - src);
+*/
 }
 
 STATIC void copyDownLine(int y)
 {
-	int size = ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS) - getScreenTop();
+/*	int size = ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS) - getScreenTop();
 	int startSize = y * SCREEN_COLUMNS;
 	
 	if (SCREEN_LINES * SCREEN_COLUMNS - size < startSize) {
 		startSize = SCREEN_LINES * SCREEN_COLUMNS - size;
 	}
 	copyDisplay(getScreenTop() + startSize, getScreenTop(), size);
+*/
 }
 
 // ץѴơ֥	
@@ -349,13 +306,13 @@ STATIC int escapeSequence(const uchar *str, const int num)
 	case '[':
 		break;
 	case 'c':
-		deleteScreen();
+		clearScreenDisplay();
 		return ESC_DONE;
 	case 'D':	// 饤󡢥뤢
-		rightCursor(SCREEN_COLUMNS);
+		rightDisplay(SCREEN_COLUMNS);
 		return ESC_DONE;
 	case 'E':	// 饤󡢥뤢
-		rightCursor(ROUNDUP(getCursorPos() + 1,SCREEN_COLUMNS));
+		rightDisplay(ROUNDUP(getCurrentDisplay() + 1, SCREEN_COLUMNS));
 		return ESC_DONE;
 	case 'M':	// 饤󥢥åסܡ饤
 		leftCursolInside(SCREEN_COLUMNS);
@@ -415,13 +372,13 @@ STATIC int escapeSequence(const uchar *str, const int num)
 			if (xy_idx == 0){
 				switch (xy[0]){
 				case 0:	// 뤫̺ǸޤǾõ
-					deleteForwardScreen(getScreenTop() + SCREEN_LINES * SCREEN_COLUMNS - getCursorPos());
+//					deleteForwardScreen(getScreenTop() + SCREEN_LINES * SCREEN_COLUMNS - getCurrentDisplay());
 					break;
 				case 1:	// ̺ǽ餫饫ޤǾõ
-					deleteBackScreen(getCursorPos() - getScreenTop());
+//					deleteBackScreen(getCurrentDisplay() - getScreenTop());
 					break;
 				case 2:	// õ
-					deleteScreen();
+					clearScreenDisplay();
 					break;
 				}
 				return ESC_DONE;
@@ -431,14 +388,14 @@ STATIC int escapeSequence(const uchar *str, const int num)
 			if (xy_idx == 0){
 				switch (xy[0]){
 				case 0:	// 뤫鱦üޤǾõ
-					deleteForwardScreen(ROUNDUP(getCursorPos() + 1, SCREEN_COLUMNS) - getCursorPos());
+					deleteForwardScreen(ROUNDUP(getCurrentDisplay() + 1, SCREEN_COLUMNS) - getCurrentDisplay());
 					break;
 				case 1:	// ü饫ޤǾõ
-					deleteBackScreen(getCursorPos() - ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS));
+					deleteBackScreen(getCurrentDisplay() - ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS));
 					break;
 				case 2:	// 饤õ
-					deleteForwardScreen(ROUNDUP(getCursorPos() + 1, SCREEN_COLUMNS) - getCursorPos());
-					deleteBackScreen(getCursorPos() - ROUNDDOWN(getCursorPos(), SCREEN_COLUMNS));
+					deleteForwardScreen(ROUNDUP(getCurrentDisplay() + 1, SCREEN_COLUMNS) - getCurrentDisplay());
+					deleteBackScreen(getCurrentDisplay() - ROUNDDOWN(getCurrentDisplay(), SCREEN_COLUMNS));
 					break;
 				}
 				return ESC_DONE;
@@ -452,7 +409,7 @@ STATIC int escapeSequence(const uchar *str, const int num)
 			return ESC_ERR;
 		case '*':	// õ
 			if (xy[0] == 0 && xy_idx == 0){
-				deleteScreen();
+				clearScreenDisplay();
 				return ESC_DONE;
 			}
 			return ESC_ERR;
@@ -475,10 +432,10 @@ STATIC INLINE int printScreen(const char c)
 {
 	switch(c){
 	case '\b':
-		leftCursor(1);
+		leftDisplay(1);
 		break;
 	case '\n':
-		newLine();
+		newLineDisplay();
 		break;
 	case '\r':
 		carriageRetun();
@@ -556,7 +513,9 @@ STATIC void printConsole(const char chr)
  * 󥽡˥Хåեñ̤ǽ񤭹
  * return : write size
  */
-int writeConsole(const char *str, size_t size)
+int writeConsole(
+	const char *str, 
+	size_t size)
 {
 	int i;
 
@@ -571,13 +530,16 @@ int writeConsole(const char *str, size_t size)
  * 󥽡˳ϡʥܡɡˤ줿饯
  * return : YES = 桼ɤ߹߲ǽˤʤä or NO = 桼ɤ߹ߤǤʤ
  */
-int sendConsole(uchar cha)
+int sendConsole(
+	const uint i_keycode)
 {
+	uint keycode = i_keycode;
+
 	// ܡɥ⥸塼ENTERͤ'\r'ʤΤ촹
-	if (cha == '\r') {
-		cha = '\n';
+	if (keycode == '\r') {
+		keycode = '\n';
 	}
-	return writeTerm(cha, &termio, &termCtl, &lbuf);
+	return writeTerm(keycode, &termio, &termCtl, &lbuf);
 }
 
 /***************************************************************************************
@@ -601,7 +563,7 @@ STATIC int read(dev_t dev, struct uio *uio, int ioflag)
 {
 	/* ե饦ɥ롼פ? */
 	while (isForeground(getCurrentProc()) == NO){
-		waitTaskSignal();
+		TaskWait();
 	}
 
 	// 󥽡Хåեɤ
@@ -639,14 +601,14 @@ STATIC int ioctl(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 			return -ENOTTY;
 		}
 		ppgid = (pid_t*) data;
-		if (checkMem(ppgid,sizeof(pid_t)) == ERR) {
+		if (vmIsReadArea(getVmProc(getCurrentProc()), ppgid, sizeof(pid_t)) == ERR) {
 			return -EFAULT;
 		}
 		*ppgid = termCtl.foregrnd;
 		break;
 	case TIOCSPGRP:			// ե饦ɥץ롼פ
 		ppgid = (gid_t*) data;
-		if (checkMem(ppgid, sizeof(gid_t)) == ERR){
+		if (vmIsReadArea(getVmProc(getCurrentProc()), ppgid, sizeof(gid_t)) == ERR){
 			return -EFAULT;
 		}
 		proc = getCurrentProc();
@@ -669,7 +631,7 @@ STATIC int ioctl(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 		break;
 	case TIOCGETA:			// termiosμ
 		tio = (struct termios*)data;
-		if (checkMem(tio,sizeof(struct termios)) == ERR){
+		if (vmIsReadArea(getVmProc(getCurrentProc()), tio,sizeof(struct termios)) == ERR){
 			return -EFAULT;
 		}
 		*tio = termio;
@@ -681,7 +643,7 @@ STATIC int ioctl(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 		// 
 	case TIOCSETAF:			// αƤϤ򥯥ꥢ٤ƤνϤδλԤätermiosꤹ
 		tio = (struct termios*)data;
-		if (checkMem(tio,sizeof(struct termios)) == ERR){
+		if (vmIsReadArea(getVmProc(getCurrentProc()), tio,sizeof(struct termios)) == ERR){
 			return -EFAULT;
 		}
 		termio = *tio;
@@ -716,7 +678,7 @@ STATIC int ioctl(dev_t dev, u_long cmd, caddr_t data, int fflag, d_thread_t *p)
 		break;
 	case TIOCGWINSZ:		// ɥμ
 		win=(struct winsize*)data;
-		if (checkMem(win,sizeof(struct winsize)) == ERR){
+		if (vmIsReadArea(getVmProc(getCurrentProc()), win,sizeof(struct winsize)) == ERR){
 			return -EFAULT;
 		}
 		win->ws_row=SCREEN_LINES;
@@ -768,7 +730,7 @@ static struct cdevsw devInfo = {
 	poll, 
 	NULL, 
 	NULL, 
-	"console", 
+	"consolek", 
 	0,
 	NULL, 
 	NULL, 
@@ -784,7 +746,7 @@ int registConsole()
 {
 	dev_t dev;
 
-	dev = make_dev(&devInfo, 0, 0, 0, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, "console");
+	dev = make_dev(&devInfo, 0, 0, 0, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP, "consolek");
 	if (dev == NODEV) {
 		return -ENODEV;
 	}
