@@ -20,7 +20,7 @@
 extern int registConsole();
 extern int initConsole();
 extern void initKeyboard();
-extern int sendConsole(uchar cha);
+extern int sendConsole(const uint);
 extern int writeConsole(const char*, size_t);
 extern int cnsoleSetTopProc();
 
