@@ -20,7 +20,7 @@ enum{
 };
 
 
-extern void initKeyboard();
+//extern void initKeyboard();
 
 
 #endif
