@@ -616,7 +616,7 @@ static int fdformat(dev, finfo, p)
 	/* set up a buffer header for fdstrategy() */
 	bp = (struct buf *)malloc(sizeof(struct buf), M_TEMP, M_NOWAIT);
 	if(bp == 0)
-		return ENOBUFS;
+		return -ENOBUFS;
 	/*
 	 * keep the process from being swapped
 	 */
@@ -644,14 +644,14 @@ static int fdformat(dev, finfo, p)
 	s = splbio();
 	while(!(bp->b_flags & B_DONE)) {
 		rv = tsleep((caddr_t)bp, PRIBIO, "fdform", 20 * hz);
-		if (rv == EWOULDBLOCK)
+		if (rv == -EWOULDBLOCK)
 			break;
 	}
 	splx(s);
 
-	if (rv == EWOULDBLOCK) {
+	if (rv == -EWOULDBLOCK) {
 		/* timed out */
-		rv = EIO;
+		rv = -EIO;
 		device_unbusy(fd->dev);
 		biodone(bp);
 	}
@@ -780,7 +780,7 @@ static int retrier(struct fdc_data *fdc)
 			}
 		}
 		bp->b_flags |= B_ERROR;
-		bp->b_error = EIO;
+		bp->b_error = -EIO;
 		bp->b_resid += bp->b_bcount - fdc->fd->skip;
 		fdc->bp = NULL;
 		fdc->fd->skip = 0;
@@ -1320,7 +1320,7 @@ static int fdc_alloc_resources(struct fdc_data *fdc)
 	fdc->res_ioport = bus_alloc_resource(dev, SYS_RES_IOPORT, &fdc->rid_ioport, 0ul, ~0ul, ispcmcia ? 8 : (ispnp ? 1 : 6), RF_ACTIVE);
 	if (fdc->res_ioport == 0) {
 		device_printf(dev, "cannot reserve I/O port range\n");
-		return ENXIO;
+		return -ENXIO;
 	}
 	fdc->portt = rman_get_bustag(fdc->res_ioport);
 	fdc->porth = rman_get_bushandle(fdc->res_ioport);
@@ -1364,7 +1364,7 @@ static int fdc_alloc_resources(struct fdc_data *fdc)
 		fdc->res_ctl = bus_alloc_resource(dev, SYS_RES_IOPORT, &fdc->rid_ctl, 0ul, ~0ul, 1, RF_ACTIVE);
 		if (fdc->res_ctl == 0) {
 			device_printf(dev, "cannot reserve control I/O port range\n");
-			return ENXIO;
+			return -ENXIO;
 		}
 		fdc->ctlt = rman_get_bustag(fdc->res_ctl);
 		fdc->ctlh = rman_get_bushandle(fdc->res_ctl);
@@ -1373,14 +1373,14 @@ static int fdc_alloc_resources(struct fdc_data *fdc)
 	fdc->res_irq = bus_alloc_resource(dev, SYS_RES_IRQ, &fdc->rid_irq, 0ul, ~0ul, 1, RF_ACTIVE);
 	if (fdc->res_irq == 0) {
 		device_printf(dev, "cannot reserve interrupt line\n");
-		return ENXIO;
+		return -ENXIO;
 	}
 
 	if ((fdc->flags & FDC_NODMA) == 0) {
 		fdc->res_drq = bus_alloc_resource(dev, SYS_RES_DRQ, &fdc->rid_drq, 0ul, ~0ul, 1, RF_ACTIVE);
 		if (fdc->res_drq == 0) {
 			device_printf(dev, "cannot reserve DMA request line\n");
-			return ENXIO;
+			return -ENXIO;
 		}
 		fdc->dmachan = fdc->res_drq->r_start;
 	}
@@ -1470,12 +1470,12 @@ int Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 
 	/* check bounds */
 	if ((fd = devclass_get_softc(fd_devclass, fdu)) == 0)
-		return (ENXIO);
+		return (-ENXIO);
 	fdc = fd->fdc;
 	if ((fdc == NULL) || (fd->type == NO_TYPE))
-		return (ENXIO);
+		return (-ENXIO);
 	if (type > NUMDENS)
-		return (ENXIO);
+		return (-ENXIO);
 	if (type == 0)
 		type = fd->type;
 	else {
@@ -1486,10 +1486,10 @@ int Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 		if (type != fd->type) {
 			switch (fd->type) {
 			case FD_360:
-				return (ENXIO);
+				return (-ENXIO);
 			case FD_720:
 				if (type != FD_820 && type != FD_800 && type != FD_640)
-					return (ENXIO);
+					return (-ENXIO);
 				break;
 			case FD_1200:
 				switch (type) {
@@ -1517,7 +1517,7 @@ int Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 					type = FD_360in5_25;
 					break;
 				default:
-					return(ENXIO);
+					return(-ENXIO);
 				}
 				break;
 			case FD_1440:
@@ -1529,7 +1529,7 @@ int Fdopen(dev_t dev, int flags, int mode, struct proc *p)
 				    && type != FD_720
 				    && type != FD_640
 				    )
-					return(ENXIO);
+					return(-ENXIO);
 				break;
 			}
 		}
@@ -1590,7 +1590,7 @@ static int fdioctl(dev, cmd, addr, flag, p)
 	fdblk = 128 << fd->ft->secsize;
 
 	switch (cmd) {
-	case DIOCGDINFO:
+/*	case DIOCGDINFO:
 		bzero(buffer, sizeof (buffer));
 		dl = (struct disklabel *)buffer;
 		dl->d_secsize = fdblk;
@@ -1600,21 +1600,21 @@ static int fdioctl(dev, cmd, addr, flag, p)
 		if (readdisklabel(dev, dl) == NULL)
 			error = 0;
 		else
-			error = EINVAL;
+			error = -EINVAL;
 
 		*(struct disklabel *)addr = *dl;
 		break;
-	case DIOCSDINFO:
+*/	case DIOCSDINFO:
 		if ((flag & FWRITE) == 0)
-			error = EBADF;
+			error = -EBADF;
 		break;
 	case DIOCWLABEL:
 		if ((flag & FWRITE) == 0)
-			error = EBADF;
+			error = -EBADF;
 		break;
-	case DIOCWDINFO:
+/*	case DIOCWDINFO:
 		if ((flag & FWRITE) == 0) {
-			error = EBADF;
+			error = -EBADF;
 			break;
 		}
 
@@ -1625,11 +1625,11 @@ static int fdioctl(dev, cmd, addr, flag, p)
 
 		error = writedisklabel(dev, (struct disklabel *)buffer);
 		break;
-	case FD_FORM:
+*/	case FD_FORM:
 		if ((flag & FWRITE) == 0)
-			error = EBADF;	/* must be opened for writing */
+			error = -EBADF;	/* must be opened for writing */
 		else if (((struct fd_formb *)addr)->format_version != FD_FORMAT_VERSION)
-			error = EINVAL;	/* wrong version of formatting prog */
+			error = -EINVAL;	/* wrong version of formatting prog */
 		else
 			error = fdformat(dev, (struct fd_formb *)addr, p);
 		break;
@@ -1639,7 +1639,7 @@ static int fdioctl(dev, cmd, addr, flag, p)
 	case FD_STYPE:                  /* set drive type */
 		/* this is considered harmful; only allow for superuser */
 		if (suser(p) != 0)
-			return EPERM;
+			return -EPERM;
 		*fd->ft = *(struct fd_type *)addr;
 		break;
 	case FD_GOPTS:			/* get drive options */
@@ -1651,11 +1651,11 @@ static int fdioctl(dev, cmd, addr, flag, p)
 	case FD_GSTAT:
 		fsp = (struct fdc_status *)addr;
 		if ((fd->fdc->flags & FDC_STAT_VALID) == 0)
-			return EINVAL;
+			return -EINVAL;
 		memcpy(fsp->status, fd->fdc->status, 7 * sizeof(u_int));
 		break;
 	default:
-		error = ENOTTY;
+		error = -ENOTTY;
 		break;
 	}
 	return (error);
@@ -1678,7 +1678,7 @@ void fdstrategy(struct buf *bp)
 	}
 	fdc = fd->fdc;
 	if (fd->type == NO_TYPE) {
-		bp->b_error = ENXIO;
+		bp->b_error = -ENXIO;
 		bp->b_flags |= B_ERROR;
 		goto bad;
 	};
@@ -1687,12 +1687,12 @@ void fdstrategy(struct buf *bp)
 	if (!(bp->b_flags & B_FORMAT)) {
 		if (bp->b_blkno < 0) {
 			printf("fd%d: fdstrat: bad request blkno = %lu, bcount = %ld\n", fdu, (u_long)bp->b_blkno, bp->b_bcount);
-			bp->b_error = EINVAL;
+			bp->b_error = -EINVAL;
 			bp->b_flags |= B_ERROR;
 			goto bad;
 		}
 		if ((bp->b_bcount % fdblk) != 0) {
-			bp->b_error = EINVAL;
+			bp->b_error = -EINVAL;
 			bp->b_flags |= B_ERROR;
 			goto bad;
 		}
@@ -1706,7 +1706,7 @@ void fdstrategy(struct buf *bp)
 		 * Reject unreasonably high block number, prevent the
 		 * multiplication below from overflowing.
 		 */
-		bp->b_error = EINVAL;
+		bp->b_error = -EINVAL;
 		bp->b_flags |= B_ERROR;
 		goto bad;
 	}
@@ -1721,7 +1721,7 @@ void fdstrategy(struct buf *bp)
 				goto bad;	/* not actually bad but EOF */
 		}
 		else {
-			bp->b_error = EINVAL;
+			bp->b_error = -EINVAL;
 			bp->b_flags |= B_ERROR;
 			goto bad;
 		}
@@ -1819,7 +1819,7 @@ static int fd_probe(device_t dev)
 
 	/* is there a unit? */
 	if (fdt == RTCFDT_NONE)
-		return (ENXIO);
+		return (-ENXIO);
 
 	/* select it */
 	set_motor(fdc, fdsu, TURNON);
@@ -1874,7 +1874,7 @@ static int fd_probe(device_t dev)
 	set_motor(fdc, fdsu, TURNOFF);
 
 	if (st0 & NE7_ST0_EC){ /* no track 0 -> no drive present */
-		return (ENXIO);
+		return (-ENXIO);
 	}
 
 	fd->track = FD_NO_TRACK;
@@ -1911,7 +1911,7 @@ static int fd_probe(device_t dev)
 		fd->type = FD_720;
 		break;
 	default:
-		return (ENXIO);
+		return (-ENXIO);
 	}
 	return (0);
 }
@@ -2037,8 +2037,8 @@ static int fdc_probe(device_t dev)
 
 	/* Check pnp ids */
 	error = ISA_PNP_PROBE(device_get_parent(dev), dev, fdc_ids);
-	if (error == ENXIO)
-		return ENXIO;
+	if (error == -ENXIO)
+		return -ENXIO;
 	if (error == 0)
 		fdc->flags |= FDC_ISPNP;
 
@@ -2054,7 +2054,7 @@ static int fdc_probe(device_t dev)
 
 	/* see if it can handle a command */
 	if (fd_cmd(fdc, 3, NE7CMD_SPECIFY, NE7_SPEC_1(3, 240), NE7_SPEC_2(2, 0), 0)) {
-		error = ENXIO;
+		error = -ENXIO;
 		goto out;
 	}
 
@@ -2137,7 +2137,7 @@ static int fdc_read_ivar(device_t dev, device_t child, int which, u_long *result
 		*result = ivars->fdunit;
 		break;
 	default:
-		return ENOENT;
+		return -ENOENT;
 	}
 	return 0;
 }
