@@ -1,4 +1,4 @@
-// ύXӏkokoŌ
+// ѹսkokoǸ
 /*-
  * Copyright (c) 1995, David Greenman
  * Copyright (c) 2001 Jonathan Lemon <jlemon@freebsd.org>
@@ -275,7 +275,7 @@ static int fxp_add_rfabuf(struct fxp_softc *sc, struct mbuf *oldm)
 
 
 /****************************************************************************
- *@heldch`
+ *ɣƣͣţģɣ
  *****************************************************************************/
 
 
@@ -326,7 +326,7 @@ fxp_ifmedia_sts(struct ifnet *ifp, struct ifmediareq *ifmr)
 }
 
 /****************************************************************************
- *@rxrbsk
+ *ӣ٣ӣãԣ
  *****************************************************************************/
 
 //koko
@@ -341,7 +341,7 @@ sysctl_int_range(SYSCTL_HANDLER_ARGS, int low, int high)
 	if (error || !req->newptr)
 		return (error);
 	if (value < low || value > high)
-		return (EINVAL);
+		return (-EINVAL);
 	*(int *)arg1 = value;
 	return (0);
 }
@@ -365,7 +365,7 @@ sysctl_hw_fxp_bundle_max(SYSCTL_HANDLER_ARGS)
 */
 
 /****************************************************************************
- *@ddoqnl
+ *ţţУңϣ
  *****************************************************************************/
 
 
@@ -544,7 +544,7 @@ fxp_write_eeprom(struct fxp_softc *sc, u_short *data, int offset, int words)
 }
 
 /****************************************************************************
- *@lhhC^[tF[X
+ *ͣɣɥ󥿡ե
  *****************************************************************************/
 
 
@@ -588,7 +588,7 @@ fxp_miibus_writereg(device_t dev, int phy, int reg, int value)
 
 
 /****************************************************************************
- *@foCXIy[V
+ *ǥХڥ졼
  ****************************************************************************/
 
 
@@ -982,7 +982,7 @@ fxp_poll(struct ifnet *ifp, enum poll_cmd cmd, int count)
 
 
 /****************************************************************************
- *@荞
+ *
  *****************************************************************************/
 
 
@@ -1164,7 +1164,7 @@ fxp_intr(void *xsc)
 }
 
 /****************************************************************************
-*@hCo[C^[tF[X
+*ɥ饤С󥿡ե
 *****************************************************************************/
 
 
@@ -1579,7 +1579,7 @@ fxp_start(struct ifnet *ifp)
 		}
 
 		if (m != NULL) {
-//koko ɂ͗Ȃ͂I
+//koko ˤʤϤ
 			ASSERT(0);
 //			struct mbuf *mn;
 
@@ -1761,14 +1761,14 @@ fxp_ioctl(struct ifnet *ifp, u_long command, caddr_t data)
 		break;
 
 	default:
-		error = EINVAL;
+		error = -EINVAL;
 	}
 	splx(s);
 	return (error);
 }
 
 /****************************************************************************
- *@
+ *
  *****************************************************************************/
 
 
@@ -1846,7 +1846,7 @@ fxp_probe(device_t dev)
 			}
 		}
 	}
-	return (ENXIO);
+	return (-ENXIO);
 }
 
 static int
@@ -1907,7 +1907,7 @@ fxp_attach(device_t dev)
 
 	if (!sc->mem) {
 		device_printf(dev, "could not map device registers\n");
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 	if (bootverbose) {
@@ -1924,7 +1924,7 @@ fxp_attach(device_t dev)
 	sc->irq = bus_alloc_resource(dev, SYS_RES_IRQ, &rid, 0, ~0, 1, RF_SHAREABLE | RF_ACTIVE);
 	if (sc->irq == NULL) {
 		device_printf(dev, "could not map interrupt\n");
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 
@@ -2109,7 +2109,7 @@ fxp_attach(device_t dev)
 	} else {
 		if (mii_phy_probe(dev, &sc->miibus, fxp_ifmedia_upd, fxp_ifmedia_sts)) {
 			device_printf(dev, "MII without any PHY!\n");
-			error = ENXIO;
+			error = -ENXIO;
 			goto fail;
 		}
 	}
@@ -2158,7 +2158,7 @@ fxp_attach(device_t dev)
 
 failmem:
 	device_printf(dev, "Failed to malloc memory\n");
-	error = ENOMEM;
+	error = -ENOMEM;
 fail:
 	splx(s);
 	fxp_release(sc);
