@@ -1,4 +1,3 @@
-// ѹսkokoǸ
 /*
  * Copyright (c) 1997, 1998
  *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
@@ -109,6 +108,8 @@
 #include <machine/bus.h>
 #include <machine/resource.h>
 #include <machine/pmap.h>		/* for vtophys */
+#include <machine/interrupt.h>
+#include <machine/lock.h>
 #include <sys/bus.h>
 #include <sys/rman.h>
 #include <dev/mii/mii.h>
@@ -175,6 +176,10 @@ static int rl_list_tx_init	__P((struct rl_softc *));
 
 #endif
 
+/*
+ * ԥåѥ
+ */
+static int spinLock;
 
 /****************************************************************************
  *ţţУңϣ
@@ -831,19 +836,26 @@ static void rl_txeof(sc)
 {
 	struct ifnet	*ifp;
 	u_int32_t		txstat;
+	int eflag;
 
 	ifp = &sc->arpcom.ac_if;
 
+	// å
+	eflag = enterCli();
+	enter_spinlock(&spinLock);
+
 	/*
 	 * Go through our tx list and free mbufs for those
 	 * frames that have been uploaded.
 	 */
 	do {
-		if (RL_LAST_TXMBUF(sc) == NULL)
+		if (RL_LAST_TXMBUF(sc) == NULL){
 			break;
+		}
 		txstat = CSR_READ_4(sc, RL_LAST_TXSTAT(sc));
-		if (!(txstat & (RL_TXSTAT_TX_OK | RL_TXSTAT_TX_UNDERRUN | RL_TXSTAT_TXABRT)))
+		if (!(txstat & (RL_TXSTAT_TX_OK | RL_TXSTAT_TX_UNDERRUN | RL_TXSTAT_TXABRT))) {
 			break;
+		}
 
 		ifp->if_collisions += (txstat & RL_TXSTAT_COLLCNT) >> 24;
 
@@ -854,8 +866,9 @@ static void rl_txeof(sc)
 		else {
 			int	oldthresh;
 			ifp->if_oerrors++;
-			if ((txstat & RL_TXSTAT_TXABRT) || (txstat & RL_TXSTAT_OUTOFWIN))
+			if ((txstat & RL_TXSTAT_TXABRT) || (txstat & RL_TXSTAT_OUTOFWIN)) {
 				CSR_WRITE_4(sc, RL_TXCFG, RL_TXCFG_CONFIG);
+			}
 			oldthresh = sc->rl_txthresh;
 			/* error recovery */
 			rl_reset(sc);
@@ -864,14 +877,23 @@ static void rl_txeof(sc)
 			 * If there was a transmit underrun,
 			 * bump the TX threshold.
 			 */
-			if (txstat & RL_TXSTAT_TX_UNDERRUN)
+			if (txstat & RL_TXSTAT_TX_UNDERRUN) {
 				sc->rl_txthresh = oldthresh + 32;
+			}
+
+			// å
+			exit_spinlock(&spinLock);
+			exitCli(eflag);
 			return;
 		}
 		RL_INC(sc->rl_cdata.last_tx);
 		ifp->if_flags &= ~IFF_OACTIVE;
 	} while (sc->rl_cdata.last_tx != sc->rl_cdata.cur_tx);
 
+	// å
+	exit_spinlock(&spinLock);
+	exitCli(eflag);
+
 	if (RL_LAST_TXMBUF(sc) == NULL)
 		ifp->if_timer = 0;
 	else if (ifp->if_timer == 0)
@@ -922,8 +944,9 @@ rl_poll (struct ifnet *ifp, enum poll_cmd cmd, int count)
 	sc->rxcycles = count;
 	rl_rxeof(sc);
 	rl_txeof(sc);
-	if (ifp->if_snd.ifq_head != NULL)
+	if (ifp->if_snd.ifq_head != NULL) {
 		rl_start(ifp);
+	}
 
 	if (cmd == POLL_AND_CHECK_STATUS) { /* also check status register */
 		u_int16_t       status;
@@ -1211,14 +1234,18 @@ static void rl_init(xsc)
  * Main transmit routine.
  */
 static void rl_start(ifp)
-	struct ifnet		*ifp;
+	struct ifnet	*ifp;
 {
 	struct rl_softc	*sc;
 	struct mbuf		*m_head = NULL;
+	int eflag;
 
 	sc = ifp->if_softc;
 
-//	while(RL_CUR_TXMBUF(sc) == NULL) {
+	// å
+	eflag = enterCli();
+	enter_spinlock(&spinLock);
+
 	if (RL_CUR_TXMBUF(sc) == NULL) {
 		IF_DEQUEUE(&ifp->if_snd, m_head);
 		if (m_head == NULL) {
@@ -1249,13 +1276,18 @@ static void rl_start(ifp)
 		m_freem(ifp->if_snd.ifq_head);
 	}
 
+	// å
+	exit_spinlock(&spinLock);
+	exitCli(eflag);
+
 	/*
 	 * We broke out of the loop because all our TX slots are
 	 * full. Mark the NIC as busy until it drains some of the
 	 * packets from the queue.
 	 */
-	if (RL_CUR_TXMBUF(sc) != NULL)
+	if (RL_CUR_TXMBUF(sc) != NULL) {
 		ifp->if_flags |= IFF_OACTIVE;
+	}
 
 	/*
 	 * Set a timeout in case the chip goes out to lunch.
@@ -1306,7 +1338,7 @@ static int rl_ioctl(ifp, command, data)
 		ifp->if_capenable = ifr->ifr_reqcap;
 		break;
 	default:
-		error = EINVAL;
+		error = -EINVAL;
 		break;
 	}
 
@@ -1371,7 +1403,7 @@ static int rl_probe(device_t dev)
 		t++;
 	}
 
-	return(ENXIO);
+	return(-ENXIO);
 }
 
 /*
@@ -1432,13 +1464,13 @@ static int rl_attach(dev)
 #ifdef RL_USEIOSPACE
 	if (!(command & PCIM_CMD_PORTEN)) {
 		printf("rl%d: failed to enable I/O ports!\n", unit);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 #else
 	if (!(command & PCIM_CMD_MEMEN)) {
 		printf("rl%d: failed to enable memory mapping!\n", unit);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 #endif
@@ -1448,7 +1480,7 @@ static int rl_attach(dev)
 
 	if (sc->rl_res == NULL) {
 		printf ("rl%d: couldn't map ports/memory\n", unit);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 	sc->rl_btag = rman_get_bustag(sc->rl_res);			// I386_BUS_SPACE_IO or I386_BUS_SPACE_MEM
@@ -1460,7 +1492,7 @@ static int rl_attach(dev)
 	if (sc->rl_irq == NULL) {
 		printf("rl%d: couldn't map interrupt\n", unit);
 		bus_release_resource(dev, RL_RES, RL_RID, sc->rl_res);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 
@@ -1508,7 +1540,7 @@ static int rl_attach(dev)
 		bus_teardown_intr(dev, sc->rl_irq, sc->rl_intrhand);
 		bus_release_resource(dev, SYS_RES_IRQ, 0, sc->rl_irq);
 		bus_release_resource(dev, RL_RES, RL_RID, sc->rl_res);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 
@@ -1519,7 +1551,7 @@ static int rl_attach(dev)
 		bus_teardown_intr(dev, sc->rl_irq, sc->rl_intrhand);
 		bus_release_resource(dev, SYS_RES_IRQ, 0, sc->rl_irq);
 		bus_release_resource(dev, RL_RES, RL_RID, sc->rl_res);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 
@@ -1535,7 +1567,7 @@ static int rl_attach(dev)
 		bus_release_resource(dev, SYS_RES_IRQ, 0, sc->rl_irq);
 		bus_release_resource(dev, RL_RES, RL_RID, sc->rl_res);
 		free(sc->rl_cdata.rl_rx_buf, M_DEVBUF);
-		error = ENXIO;
+		error = -ENXIO;
 		goto fail;
 	}
 
