@@ -5,17 +5,13 @@
  * Distributed under the terms of the BSD License.
  */
 
-
 #ifndef i386_DevInterrupt_h
 #define i386_DevInterrupt_h
 
-
 #include <i386/syscall.h>
 
-
 #ifndef ASM_FILE
 
-
 enum {
 	// IRQȥ꡼ơ֥
 	IRQ0			= 0,
@@ -43,14 +39,8 @@ enum {
 extern int (*irq_entry[])(int);
 extern void (*release_irq_mask)(uchar);
 extern void (*set_irq_mask)(uchar);
-extern void (*irq_eoi)(uchar);
-extern void *changeIntrTask();
-extern void *endIntrTask();
-extern void setUserIntrHandler(const int);
-extern void setKernIntrHandler(const int);
-
+extern void (*irq_eoi)(const int);
 
 #endif
 
-
 #endif
