@@ -10,20 +10,16 @@
 #ifndef _MACHINE_CLOCK_H_
 #define	_MACHINE_CLOCK_H_
 
-
+#include <sys/types.h>
 #include <lib/lib.h>
 
-
 #ifdef KERNEL
 
-
 #define DELAY micro_timer
 
-
+extern uint64 getCpuClock();
 extern int	rtcin __P((int val));
 
-
 #endif /* KERNEL */
 
-
 #endif /* !_MACHINE_CLOCK_H_ */
