@@ -9,10 +9,8 @@
 #ifndef i386_lib_h
 #define i386_lib_h
 
-
 #ifndef ASM_FILE
 
-
 static inline u_int bsfl(u_int mask)
 {
 	u_int	result;
@@ -34,7 +32,8 @@ static inline int ffs(int mask)
 
 extern void reboot();
 extern void saveRealIdt();
-
+extern int getBuddyIndex(const uint);
+extern uint getBuddySize(const int);
 
 #endif
 
