@@ -12,16 +12,16 @@
 #include <sys/param.h>
 #include <sys/config.h>
 #include <lib/lib.h>
+#include <lib/bitmap.h>
 #include <dev/console/console.h>
 #include <dev/MC146818.h>
 #include <dev/8254.h>
 #include <dev/pci/pci.h>
-#include <kern/errno.h>
 #include <kern/time.h>
-#include <kern/lock.h>
-#include <kern/Thread.h>
+#include <sys/Thread.h>
 #include <kern/time.h>
 #include <kern/vm.h>
+#include <i386/lock.h>
 #include <i386/pcibus.h>
 #include <i386/fpu.h>
 #include <i386/tss.h>
@@ -30,6 +30,7 @@
 #include <i386/Entry.h>
 #include <i386/sp.h>
 #include <i386/DevInterrupt.h>
+#include <i386/clock.h>
 #include <i386/mp.h>
 
 #include <kern/debug.h>
@@ -113,12 +114,12 @@ enum{
 	LOCAL_APIC_LDR	= 0xd0 / sizeof(int),
 	LOCAL_APIC_DFR	= 0xe0 / sizeof(int),
 	LOCAL_APIC_SVR	= 0xf0 / sizeof(int),
-	LOCAL_APIC_ISR1	= 0x100 / sizeof(int),
 	LOCAL_APIC_ISR2	= 0x110 / sizeof(int),
+	LOCAL_APIC_ISR3	= 0x120 / sizeof(int),
 	LOCAL_APIC_TMR1	= 0x180 / sizeof(int),
 	LOCAL_APIC_TMR2	= 0x190 / sizeof(int),
 	LOCAL_APIC_IRR1	= 0x200 / sizeof(int),
-	LOCAL_APIC_IRR2	= 0x200 / sizeof(int),
+	LOCAL_APIC_IRR2	= 0x210 / sizeof(int),
 	LOCAL_APIC_ICR1	= 0x300 / sizeof(int),
 	LOCAL_APIC_ICR2	= 0x310 / sizeof(int),
 	LOCAL_APIC_TIME	= 0x320 / sizeof(int),
@@ -203,24 +204,68 @@ extern int *active_cpu;		/* ưCPU */
 
 static uint MFPS_addres;
 
+static uchar ioapic_intr_vect[]=
+{
+	IRQ0_VECT ,
+	IRQ1_VECT ,
+	0         ,
+	IRQ3_VECT ,
+	IRQ4_VECT ,
+	IRQ5_VECT ,
+	IRQ6_VECT ,
+	IRQ7_VECT ,
+	IRQ8_VECT ,
+	IRQ9_VECT ,
+	IRQ10_VECT,
+	IRQ11_VECT,
+	IRQ12_VECT,
+	IRQ13_VECT,
+	IRQ14_VECT,
+	IRQ15_VECT,
+	APIC_TIMER_VECT,
+	FLASH_PAGEDIR_VECT,
+};
+
 //--------------------------------------------------------------------------------------------------
 // LOCAL APIC
 //--------------------------------------------------------------------------------------------------
 
-static uint volatile *local_apic = (uint*)LOCAL_APIC_BASE;	/* LOCAL APIC base address */
-static uint task_count;										/* 1ư֤APICޡ */
+static uint volatile *local_apic = (uint*) LOCAL_APIC_BASE;		// LOCAL APIC base address
+static uint task_count;											// 1ư֤APICޡ
+static int64 waitEoi[MAX_CPU];									// EOIαߥ٥ӥåȥޥå
 
 /*
  * ϡɳEOIؿ
+ *ճ߶ػߤǸƤФ
  */
-STATIC void mpEoi(
-	uchar dummy)
+static void mpEoi(
+	const int irq)
 {
-//*********************************************************
-//testDevInterrupt(dummy, -1, 0);
-//*********************************************************
-	/* issue EOI */
+	int cpu = getCpu();
+	int64 isrBitMap = ((int64) local_apic[LOCAL_APIC_ISR3] << 32) + local_apic[LOCAL_APIC_ISR2];
+	int bitPos;
+
+	bitPos = ioapic_intr_vect[irq] - 32;
+
+	// ߥϥɥϽǤϤ
+	ASSERT(isrBitMap & ((int64) 1 << bitPos));
+
+	bitmapReset(bitPos, 1, (u_int8_t*) &isrBitMap);
+
+	// ͥ٤ι⤤ߥϥɥ餬ʤEOIȯα
+	if (bitPos < getBitmapPos(bitPos, 64 - bitPos, (u_int8_t*) &isrBitMap)) {
+		bitmapSet(bitPos, 1, (u_int8_t*) &waitEoi[cpu]);
+		return;
+	}
+
+	// EOIȯ
 	local_apic[LOCAL_APIC_EOI] = 0;
+
+	// αʬEOIȯ
+	for (bitPos = getBitmapPos(0, 64, (u_int8_t*) &waitEoi[cpu]); bitPos != ERR; bitPos = getBitmapPos(0, 64, (u_int8_t*) &waitEoi[cpu])) {
+		local_apic[LOCAL_APIC_EOI] = 0;
+		bitmapReset(bitPos, 1, (u_int8_t*) &waitEoi[cpu]);
+	}
 }
 
 /*
@@ -247,7 +292,7 @@ STATIC uint count_apictimer()
 STATIC int apictimer_handler(
 	const int dummy)
 {
-/*****************************************************************************/
+#ifdef DEBUG
 	{
 		static char value[MAX_CPU] = {'0','0','0','0','0','0','0','0'};
 		int cpu = getCpu();
@@ -258,7 +303,7 @@ STATIC int apictimer_handler(
 		displayTop(50 + cpu, &value[cpu],1);
 		++value[cpu];
 	}
-/****************************************************************************/
+#endif
 	// restart APIC TIMER
 	local_apic[LOCAL_APIC_ICNT] = task_count;
 
@@ -312,7 +357,7 @@ STATIC void init_localapic(
 	local_apic[LOCAL_APIC_TDVC] = 0xb;						/* ʬ1 */
 	local_apic[LOCAL_APIC_TIME] = APIC_TIMER_VECT;			/* 󥷥åȥ⡼ | ޥ */
 	if (cpu == 0) {
-		setDevIntrEntry(APIC_TIMER_VECT, APIC_TIMER, doKernIntrHandler, mpEoi);
+		setDevIntrEntry(APIC_TIMER_VECT, APIC_TIMER, mpEoi);
 		value = count_apictimer();							/* APICޡHZ¬ */
 		printk("APIC TIMER = %d HZ\n", value);
 		task_count = value / (1000 / TASK_TIME);			/* ư */
@@ -385,25 +430,6 @@ static uint ioapic_hrt[]=
 	IO_APIC_HRT18,
 	IO_APIC_HRT19,
 };
-static uchar ioapic_intr_vect[]=
-{
-	IRQ0_VECT ,
-	IRQ1_VECT ,
-	0         ,
-	IRQ3_VECT ,
-	IRQ4_VECT ,
-	IRQ5_VECT ,
-	IRQ6_VECT ,
-	IRQ7_VECT ,
-	IRQ8_VECT ,
-	IRQ9_VECT ,
-	IRQ10_VECT,
-	IRQ11_VECT,
-	IRQ12_VECT,
-	IRQ13_VECT,
-	IRQ14_VECT,
-	IRQ15_VECT,
-};
 static uint volatile *io_apic_regsel=(uint*)IO_APIC_REGSEL;	/* IOAPIC REGSEL address */
 static uint volatile *io_apic_win=(uint*)IO_APIC_WIN;		/* IOAPIC WIN address */
 
@@ -477,21 +503,21 @@ STATIC void init_ioapic(
 	write_ioapic(IO_APIC_ID, cpu << 24);
 
 	// ߥȥ꡼
-	setDevIntrEntry(IRQ0_VECT,  IRQ0,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ1_VECT,  IRQ1,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ3_VECT,  IRQ3,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ4_VECT,  IRQ4,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ5_VECT,  IRQ5,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ6_VECT,  IRQ6,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ7_VECT,  IRQ7,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ8_VECT,  IRQ8,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ9_VECT,  IRQ9,  doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ10_VECT, IRQ10, doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ11_VECT, IRQ11, doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ12_VECT, IRQ12, doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ13_VECT, IRQ13, doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ14_VECT, IRQ14, doKernIntrHandler, mpEoi);
-	setDevIntrEntry(IRQ15_VECT, IRQ15, doKernIntrHandler, mpEoi);
+	setDevIntrEntry(IRQ0_VECT,  IRQ0,  mpEoi);
+	setDevIntrEntry(IRQ1_VECT,  IRQ1,  mpEoi);
+	setDevIntrEntry(IRQ3_VECT,  IRQ3,  mpEoi);
+	setDevIntrEntry(IRQ4_VECT,  IRQ4,  mpEoi);
+	setDevIntrEntry(IRQ5_VECT,  IRQ5,  mpEoi);
+	setDevIntrEntry(IRQ6_VECT,  IRQ6,  mpEoi);
+	setDevIntrEntry(IRQ7_VECT,  IRQ7,  mpEoi);
+	setDevIntrEntry(IRQ8_VECT,  IRQ8,  mpEoi);
+	setDevIntrEntry(IRQ9_VECT,  IRQ9,  mpEoi);
+	setDevIntrEntry(IRQ10_VECT, IRQ10, mpEoi);
+	setDevIntrEntry(IRQ11_VECT, IRQ11, mpEoi);
+	setDevIntrEntry(IRQ12_VECT, IRQ12, mpEoi);
+	setDevIntrEntry(IRQ13_VECT, IRQ13, mpEoi);
+	setDevIntrEntry(IRQ14_VECT, IRQ14, mpEoi);
+	setDevIntrEntry(IRQ15_VECT, IRQ15, mpEoi);
 
 	/* IOơ֥,All maske */
 	write_ioapic(IO_APIC_LRT0 ,IOREDTBL_MASK | IOREDTBL_MODE_LOGIC | IOREDTBL_DELIV_LOWPRT | IRQ0_VECT);
@@ -510,8 +536,8 @@ STATIC void init_ioapic(
 	write_ioapic(IO_APIC_HRT6 ,0xff000000);                                                 
 	write_ioapic(IO_APIC_LRT7 ,IOREDTBL_MASK | IOREDTBL_MODE_LOGIC | IOREDTBL_DELIV_LOWPRT | IRQ7_VECT);
 	write_ioapic(IO_APIC_HRT7 ,0xff000000);                                                 
-	write_ioapic(IO_APIC_LRT8 ,IOREDTBL_MASK | IOREDTBL_MODE_LOGIC | IOREDTBL_DELIV_LOWPRT | IRQ8_VECT);
-	write_ioapic(IO_APIC_HRT8 ,0xff000000);                                                 
+	write_ioapic(IO_APIC_LRT8 ,IOREDTBL_MASK | IOREDTBL_MODE_PHYS | IOREDTBL_DELIV_FIX | IRQ8_VECT);
+	write_ioapic(IO_APIC_HRT8 ,0x00000000);                                                 
 	write_ioapic(IO_APIC_LRT9 ,IOREDTBL_MASK | IOREDTBL_MODE_LOGIC | IOREDTBL_DELIV_LOWPRT | IRQ9_VECT);
 	write_ioapic(IO_APIC_HRT9 ,0xff000000);                                                 
 	write_ioapic(IO_APIC_LRT10,IOREDTBL_MASK | IOREDTBL_MODE_LOGIC | IOREDTBL_DELIV_LOWPRT | IRQ10_VECT);
@@ -649,11 +675,11 @@ int init_mp()
 	boot_acpu();
 
 	// pagingǻȤpage directoryߤ
-	setDevIntrEntry(FLASH_PAGEDIR_VECT, FLASH_PAGEDIR, doKernIntrHandler, mpEoi);
+	setDevIntrEntry(FLASH_PAGEDIR_VECT, FLASH_PAGEDIR, mpEoi);
 	irq_entry[FLASH_PAGEDIR] = flash_pagedir_handler;
 
 	/* Init IO apic٤Ƥcpuεưȯ */
-	beginClock = rdtsc();
+	beginClock = getCpuClock();
 	while (*active_cpu != cpu_num) {
 		if (WAIT_ACTIVE_CPU < getMiliSecondsFromClock(beginClock)) {
 			printk("Booting application cpu isn't complete!\n");
@@ -691,7 +717,7 @@ void init_application_cpu()
 		initSchedule(cpu);
 
 		/* cpu clockη¬ */
-		printk("cpu%d clock = %d HZ\n", cpu, getCpuClock());
+		printk("cpu%d clock = %d HZ\n", cpu, calcCpuClock());
 
 		++*active_cpu;
 	}
@@ -705,7 +731,7 @@ void init_application_cpu()
 		Cpu cpu;
 
 		if (CpuConstructor(&cpu) != NOERR) {
-			PRINT_ERR("This cpu is not supported!");
+			panic("This cpu is not supported file=%s line=%d", __FILE__, __LINE__);
 		}
 		if (cpu.isSupportSysenter(&cpu) == YES) {
 			cpu.setSysenter(&cpu);
@@ -778,6 +804,7 @@ void setPciIrqInIoapic(const int bus, const int device, const int intpin, const
 	pciIrq += IO_APIC_PCIIRQ;
 
 	write_ioapic(ioapic_lrt[pciIrq], read_ioapic(ioapic_lrt[pciIrq]) | ioapic_intr_vect[irq]);
+
 	// ߥޥ
 	write_ioapic(ioapic_lrt[pciIrq], read_ioapic(ioapic_lrt[pciIrq]) & ~IOREDTBL_MASK);
 }
@@ -825,7 +852,8 @@ void set_logical_intr(int irq)
 
 //*************************************************************************************************
 
-void test_mp()
+void testMp(
+	const int irq,
+	void *task)
 {
-	printk("IRQ14REG LOW=%x HIGH=%x\n",read_ioapic(IO_APIC_LRT14),read_ioapic(IO_APIC_HRT14));
 }
