@@ -32,17 +32,18 @@ typedef unsigned int		u32;
 typedef unsigned long long	u64;
 
 typedef uint32_t			mem_addr_t;		// ꡼ɥ쥹
-
 typedef	int					intptr_t;
 typedef	unsigned int		uintptr_t;
-
 typedef mem_addr_t			ptrdiff_t;		/* ptr1 - ptr2 */
-
 typedef	mem_addr_t			vm_offset_t;
 typedef uint32_t			intrmask_t;
-
 typedef	void				inthand2_t (void *_cookie);
 
+typedef unsigned long		paddr_t;
+typedef unsigned long		psize_t;
+typedef unsigned long		vsize_t;
+typedef unsigned long		vaddr_t;
+
 #define BITS_PER_LONG 32
 
 
