@@ -15,20 +15,19 @@
 #include <sys/config.h>
 #include <sys/types.h>
 #include <sys/param.h>
-#include <sys/AggregateList.h>
+#include <lib/AggregateList.h>
 #include <sys/sched.h>
 #include <sys/pthread.h>
 #include <machine/Cpu.h>
 #include <machine/fpu.h>
 #include <machine/mp.h>
-#include <machine/UserHandler.h>
+#include <machine/clock.h>
+#include <machine/lock.h>
 #include <machine/interrupt.h>
 #include <kern/kmalloc.h>
-#include <kern/time.h>
-#include <kern/lock.h>
 #include <kern/itimer.h>
-#include <kern/Wait.h>
-#include <kern/Thread.h>
+#include <kern/TaskWait.h>
+#include <sys/Thread.h>
 
 #include <kern/debug.h>
 
@@ -49,11 +48,12 @@
  * åɥ塼¤
  */
 typedef struct {
-	int cpu;					// CPUֹ 0
-	void *currentTask;			// ߼¹ΥåID
-	uint taskCount;				// ưåɿ
-	AggregateList aggregate;	// åɥ塼ꥹȽ
-	int lock;					// 塼¾å
+	int cpu;						// CPUֹ 0
+	void *currentTask;				// ߼¹ΥåID
+	uint taskCount;					// ưåɿ
+	AggregateList aggregate;		// åɥ塼ꥹȽ
+	AggregateListMethod aggrMethod;	// ꥹȥ᥽å
+	int lock;						// 塼¾å
 } SCHEDULE;
 
 /*
@@ -62,19 +62,19 @@ typedef struct {
 typedef struct Thread {
 //================ Ϣ ===========================
 	List			list;				// 塼ѥꥹ
-	List			waitList;			// ѥꥹ
-	WaitObj			waitTask;			// ȥ֥
+	TaskWaitObj		taskWaitObj;		// ȥ֥
 	VmTaskObj		vmTaskObj;			// ۥ꡼֥
 	int				nestCount;			// ͥͥȥ
 	int				cpu;				// running cpu number
 	int				state;				// 
+	int 			fpu;				// FPU֥ե饰᥻֤
 
 	struct Thread	*callTask;			// ƤӽФ西
 	struct Thread	*callDstTask;		// ƤӽФ
 	int				irq;				// ɥ饤С⥸塼ѳֹ
-	uint			taskBuf[TASK_PARAM_BUF];	// åѥ᡼Хåե
 
 	TaskSignalObj	taskSignalObj;		// ʥ¤
+	EntryObj		entryObj;			// ȥ꡼¤
 
 //================ åɴϢ ===========================
 	pthread_t		id;					// ID
@@ -99,7 +99,10 @@ typedef struct Thread {
 extern int registThread(void*, List*);
 extern int procGetThread(const pthread_t, ThreadObj**);
 extern int procGetWaitThread(const pthread_t, ThreadObj**);
-extern void waitConstruct(WaitObj*, ThreadObj*);
+extern void taskWaitConstruct(TaskWaitObj*, ThreadObj*);
+#ifdef DEBUG
+extern void displayTask(ThreadObj*, const int);
+#endif
 
 //===================================== PRIVATE ====================================================
 
@@ -152,13 +155,16 @@ STATIC int addSchedule(
 	const int cpu,
 	const int state)
 {
-	// ߤΥȥե饰ȰפƤ뤫
+	/*
+	 * ߤΥȥե饰ȰפƤɲä
+	 */
 	if (m_task->state & state) {
 		AggregateList *aggregate = &schedule[cpu].aggregate;
-		Thread *current = aggregate->refHead(aggregate);
+		AggregateListMethod *aggrMethod = &schedule[cpu].aggrMethod;
+		Thread *current = aggrMethod->refHead(aggregate);
 
 		m_task->state = TASK_RUNNING;
-		aggregate->insertNext(aggregate, &current->list, &m_task->list);
+		aggrMethod->insertNext(aggregate, &current->list, &m_task->list);
 
 		// ɥ륹åɤϥȥåפ
 		if(current == idleTask[cpu]) {
@@ -182,17 +188,18 @@ STATIC void delSchedule(
 	int cpu = getCpu();
 
 	AggregateList *aggregate = &schedule[cpu].aggregate;
-	Thread *current = aggregate->refHead(aggregate);
+	AggregateListMethod *aggrMethod = &schedule[cpu].aggrMethod;
+	Thread *current = aggrMethod->refHead(aggregate);
 
 	/*
 	 * ¹ԥʤʤϥɥ륿³
 	 */
-	aggregate->removeEntry(aggregate, &current->list);
-	if (aggregate->refHead(aggregate) == NULL) {
-		aggregate->insertHead(aggregate, &idleTask[cpu]->list);
-	}
-
+	aggrMethod->removeEntry(aggregate, &current->list);
 	current->state = state;
+	if (aggrMethod->refHead(aggregate) == NULL) {
+		aggrMethod->insertHead(aggregate, &idleTask[cpu]->list);
+		idleTask[cpu]->state = TASK_RUNNING;
+	}
 }
 
 STATIC INLINE Thread *currentTask()
@@ -216,12 +223,12 @@ STATIC void threadConstructor(
 	memset(this, 0, sizeof(*this));
 	this->id = id++;
 	listConstructor(&this->list, this);
-	listConstructor(&this->waitList, this);
 	listConstructor(&this->procList, this);
-	waitConstruct(&this->waitTask, (ThreadObj*) this);
+	taskWaitConstruct(&this->taskWaitObj, (ThreadObj*) this);
 	this->proc = i_proc;
 
 	initProcessTimer(&this->timer);
+	EntryConstructor(&this->entryObj);
 }
 
 /*
@@ -230,8 +237,8 @@ STATIC void threadConstructor(
 STATIC void threadDestructor(
 	Thread *this)
 {
-	// ޡγ
 	releaseProcessTimer(&this->timer);
+	EntryDestructor(&this->entryObj);
 
 	kfree(this);
 }
@@ -250,7 +257,7 @@ STATIC void wakeExitWaitThread(
 
 	// λԤåɤ򵯤
 	if (procGetWaitThread(this->id, (ThreadObj**) &waitThread) == NOERR) {
-		activeTaskSignal(getWaitTask((ThreadObj*) waitThread));
+		TaskAwake(getTaskWait((ThreadObj*) waitThread));
 	}
 }
 /*
@@ -289,7 +296,7 @@ STATIC int waitJoinThread(
 
 	// åɽλԤ
 	while (joinThread->valuePtr == NULL) {
-		waitTaskSignal();
+		TaskWait();
 		
 		// λʥʤ齪λ
 		if (isEndSigint(TaskGetTaskSignal(getCurrentTask())) == YES) {
@@ -317,7 +324,7 @@ void initSchedule(
 	int cpu)
 {
 	schedule[cpu].cpu = cpu;
-	AggregateCircleConstructor(&schedule[cpu].aggregate);
+	AggregateCircleConstructor(&schedule[cpu].aggregate, &schedule[cpu].aggrMethod);
 }
 
 /*
@@ -410,12 +417,30 @@ void setFirstTask(
 {
 	Thread *task = (Thread*) m_task;
 
-	schedule[i_cpu].aggregate.insertHead(&schedule[i_cpu].aggregate, &task->list);
+	schedule[i_cpu].aggrMethod.insertHead(&schedule[i_cpu].aggregate, &task->list);
 	schedule[i_cpu].currentTask = task;
 	task->cpu = i_cpu;
 	task->state = TASK_RUNNING;
 }
 
+/*
+ * 󥿥¹ԥ塼³ȥ˥å
+ *ճ߶ػߤǸƤФ
+ */
+void addScheduleDaemon(
+	ThreadObj *i_task)
+{
+	Thread *task = (Thread*) i_task;
+	int cpu = getCpu();
+
+	task->cpu = cpu;
+	enter_spinlock(&schedule[cpu].lock);
+	{
+		addSchedule(task, cpu, TASK_INTR_WAIT);
+	}
+	exit_spinlock(&schedule[cpu].lock);
+}
+
 /*
  * 塼륿Ѥ
  *ռƤӽФ
@@ -427,14 +452,15 @@ void changeTask(
 	Thread *task = (Thread*) m_task;
 	int cpu = getCpu();
 	AggregateList *aggregate = &schedule[cpu].aggregate;
+	AggregateListMethod *aggrMethod = &schedule[cpu].aggrMethod;
 
 	enter_spinlock(&schedule[cpu].lock);
 	{
-		Thread *current = aggregate->refHead(aggregate);
+		Thread *current = aggrMethod->refHead(aggregate);
 
-		aggregate->removeEntry(aggregate, &current->list);
+		aggrMethod->removeEntry(aggregate, &current->list);
 		current->state = TASK_WAIT;
-		aggregate->insertHead(aggregate, &task->list);
+		aggrMethod->insertHead(aggregate, &task->list);
 		task->cpu = cpu;
 		task->state = TASK_RUNNING;
 	}
@@ -443,18 +469,19 @@ void changeTask(
 
 /*
  * å
- *ճ߶ػߤǸƤФ
+ *ճ߶ػߤǸƤФ뤳
  * return : ΥΥڡǥ쥯ȥ
  */
 void *switchTask()
 {
 	int cpu = getCpu();
 	AggregateList *aggregate = &schedule[cpu].aggregate;
+	AggregateListMethod *aggrMethod = &schedule[cpu].aggrMethod;
 	Thread *next;
 
 	// Υ˥å
-	aggregate->pointHeadNext(aggregate);
-	next = aggregate->refHead(aggregate);
+	aggrMethod->pointHeadNext(aggregate);
+	next = aggrMethod->refHead(aggregate);
 	schedule[cpu].currentTask = next;
 
 	return getPageDir(&next->vmTaskObj);
@@ -508,23 +535,23 @@ VmTaskObj *getVmTask(
 }
 
 /*
- * wait list 
+ * ȥ꡼¤Τ
  */
-List *getWaitList(
+EntryObj *getEntry(
 	ThreadObj *i_this)
 {
 	Thread *this = (Thread*) i_this;
-	return &this->waitList;
+	return &this->entryObj;
 }
 
 /*
  * ȥ֥Ȥ
  */
-WaitObj *getWaitTask(
+TaskWaitObj *getTaskWait(
 	ThreadObj *i_this)
 {
 	Thread *this = (Thread*) i_this;
-	return &this->waitTask;
+	return &this->taskWaitObj;
 }
 
 /*
@@ -560,16 +587,6 @@ ThreadObj *getCallDstTask(
 	return (ThreadObj*) this->callDstTask;
 }
 
-/*
- * åɥѥ᡼Хåեμ
- */
-void *getParamBuf(
-	ThreadObj *i_this)
-{
-	Thread *this = (Thread*) i_this;
-	return this->taskBuf;
-}
-
 /*
  * ɥ饤С⥸塼ֹ
  *ռƤӽФ
@@ -665,12 +682,12 @@ TaskSignalObj *TaskGetTaskSignal(
 
 /*
  *ճߥͥȥȲûޤϸ˸ƤӽФ
- * ץַ¬rdtsc͡ˤꤹ
+ * ץַ¬getCpuClock͡ˤꤹ
  */
 void setTaskCurrentTime()
 {
 	if (currentTask()->nestCount == 1) {
-		currentTask()->beforeClocks = rdtsc();
+		currentTask()->beforeClocks = getCpuClock();
 	}
 }
 
@@ -683,7 +700,7 @@ void calcTaskUserTime()
 	Thread *current = currentTask();
 
 	if (current->nestCount == 1) {
-		int64 userClocks = rdtsc() - current->beforeClocks;
+		int64 userClocks = getCpuClock() - current->beforeClocks;
 
 		current->userClocks += userClocks;
 		doItimer(userClocks, SIGVTALRM, current->proc, getItimerVirtual());
@@ -699,7 +716,7 @@ void calcTaskSysTime()
 	Thread *current = currentTask();
 
 	if (current->nestCount == 1) {
-		int64 sysClocks = rdtsc() - current->beforeClocks;
+		int64 sysClocks = getCpuClock() - current->beforeClocks;
 		
 		current->sysClocks += sysClocks;
 
@@ -834,6 +851,7 @@ int createIdleThread(
 
 /*
  * åɤե
+ *ԳԲġ
  * return : error number or 1 = 
  */
 int forkThread(
@@ -909,11 +927,11 @@ int sys_pthread_create(
 	void *proc;
 	int error;
 
-	if (checkMem(o_thread, sizeof(*o_thread)) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), o_thread, sizeof(*o_thread)) == ERR) {
 		return -EFAULT;
 	}
 	if (attr != NULL) {
-		if (checkMem(attr, sizeof(*attr)) == ERR) {
+		if (vmIsReadArea(getVmProc(getCurrentProc()), attr, sizeof(*attr)) == ERR) {
 			return -EFAULT;
 		}
 	}
@@ -980,24 +998,96 @@ int sys_pthread_join(
 	return waitJoinThread(this, i_joinId, o_valuePtr);
 }
 
+int sys_pthread_self(
+	pthread_t *o_thread)
+{
+	Thread *thread = (Thread*) getCurrentTask();
+	*o_thread = thread->id;
+
+	return 0;
+}
+
+/*
+ * åɾμ
+ */
+int sysPthreadGetState(
+	pthread_t id,
+	struct pthreadState *m_state)
+{
+	Thread *thread;
+	int error;
+	
+	error = procGetThread(id, (ThreadObj**) &thread);
+	if (error != NOERR) {
+		return error;
+	}
+	m_state->taskState = thread->state;
+
+	return NOERR;
+}
+
+/*
+ * åɾμ
+ */
+int sysGetTask(
+	void **o_task)
+{
+	*o_task = getCurrentTask();
+
+	return NOERR;
+}
+
 //**************************************************************************************************
 
 #ifdef DEBUG
-void displaySchedule(
-	const int cpu)
-{
-	AggregateList *aggregate = &schedule[cpu].aggregate;
-	IteratorList iterator;
-	int i;
+#include <lib/IteratorList.h>
 
-	displayTop(80 * (10 + cpu) + 60, "                    ", 20);
-	aggregate->iterator(aggregate, &iterator);
-	i = 0;
-	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-//		printk("%x ", iterator.next(&iterator));
-		printDebug(80 * (10 + cpu) + 60 + i * 10, "%x ", iterator.next(&iterator));
-		++i;
+enum {
+	MONITOR_NUM		= 3,
+	CPU_NUM			= 2,
+	TASK_NUM		= 5,
+};
+
+static void *task[MONITOR_NUM][CPU_NUM][TASK_NUM];
+
+void monitorSchedule(
+	const int num)
+{
+	int cpu;
+	IteratorList iterator;
+
+	for (cpu = 0; cpu < CPU_NUM; ++cpu) {
+		int i = 0;
+
+		IteratorListConstruct(&iterator, &schedule[cpu].aggregate, &schedule[cpu].aggrMethod);
+		while (iterator.hasNext(&iterator) == BOOL_TRUE) {
+			task[num][cpu][i++] = iterator.next(&iterator);
+			if (i == TASK_NUM) {
+				break;
+			}
+			task[num][cpu][i] = NULL;
+		}
+	}
+}
+
+void displaySchedule()
+{
+	int num;
+	int cpu;
+
+	for (num = 0; num < MONITOR_NUM; ++num) {
+		for (cpu = 0; cpu < CPU_NUM; ++cpu) {
+			int i;
+			
+			printk("cpu%d", cpu);
+			for (i = 0; i < TASK_NUM; ++i) {
+				if (task[num][cpu][i] == NULL) {
+					break;
+				}
+				printk(" %x", task[num][cpu][i]);
+			}
+			printk("\n");
+		}
 	}
-//	printk("\n");
 }
 #endif
