@@ -5,32 +5,26 @@
  * Distributed under the terms of the BSD License.
  */
 
-
-#ifndef devfs_h
-#define devfs_h
-
-
-#include <kern/dev_common.h>
-
+#ifndef kern_devfs_h
+#define kern_devfs_h
 
 #ifndef ASM_FILE
 
-
 #define DEV_FS_NAME "devFs"
 
 enum{
-	DEVICE_ROOT_INODE	= 0,				// ǥХե륷ƥ롼inodeֹ
+	DEVICE_ROOT_INODE	= 0,	// ǥХե륷ƥ롼inodeֹ
 };
 
-
 extern int initDeviceFs();
 extern int makeDevf(const char *, void *, const uint);
-extern int openDevice(const char *devPath, void**);
+extern int makeDevfModule(const char *, void *, const uint);
 extern void *getDevInfo(const void*);
 extern int getDevInodeNum(const void *);
 extern int getTtyname(void *, char *);
+extern int devOpen(const char *devPath, void**);
+extern int devIoctl(void*, int, caddr_t);
 
-#endif
+#endif // ASM_FILE
 
-
-#endif
+#endif // kern_devfs_h
