@@ -11,60 +11,55 @@
 #include <sys/param.h>
 #include <sys/errno.h>
 #include <sys/systm.h>
+#include <sys/conf.h>
 #include <sys/uio.h>
-#include <sys/AggregateList.h>
+#include <lib/IteratorList.h>
+#include <lib/AggregateList.h>
 #include <lib/lib.h>
 #include <lib/lib_path.h>
 #include <machine/interrupt.h>
+#include <machine/lock.h>
+#include <module/Module.h>
 #include <kern/kmalloc.h>
-#include <kern/lock.h>
 #include <kern/timer.h>
 #include <kern/ProcSignal.h>
-#include <kern/block_cache.h>
+#include <kern/BlockCache.h>
 #include <kern/devfs.h>
-#include <kern/dev_common.h>
-#include <kern/Wait.h>
+#include <kern/TaskWait.h>
 #include <kern/device.h>
 
 #include <kern/debug.h>
 
-
-//#define DEBUG_DEVICE 1
-#ifdef DEBUG_DEVICE
-	#define STATIC
-	#define INLINE
-#else
-	#define STATIC static
-	#define INLINE inline
-#endif
-
-
-//================================== PRIVATE ============================================
+//=====================================  ===================================================
 
 /*
  * ǥХ¤Υꥹȥإå
  */
 typedef struct {
-	AggregateList	aggregate;		// ǥХꥹȥإå
-	int				lock;			// ԥå
+	AggregateList		aggregate;		// ǥХꥹȥإå
+	AggregateListMethod	aggrMethod;		// ꥹȥ᥽å
+	int					lock;			// ԥå
 } DEV_SPEC_HEAD;
 
+//===================================== Х륤ݡ =======================================
+
+//================================== PRIVATE ============================================
+
 static DEV_SPEC_HEAD devSpecHead;
 
 /*
  * ǥХ̾õ
  * return : struct specinfo or NULL
  */
-STATIC struct specinfo *searchDevByName(
+static struct specinfo *searchDevByName(
 	const char *i_name)
 {
-	AggregateList *aggregate = &devSpecHead.aggregate;
 	IteratorList iterator;
 	struct specinfo *dev;
 
 	enter_spinlock(&devSpecHead.lock);
 	{
-		aggregate->iterator(aggregate, &iterator);
+		IteratorListConstruct(&iterator, &devSpecHead.aggregate, &devSpecHead.aggrMethod);
 		for (; ; ) {
 			if (iterator.hasNext(&iterator) == BOOL_FALSE) {
 				dev = NULL;
@@ -86,16 +81,15 @@ STATIC struct specinfo *searchDevByName(
  * ǥХ̾õ
  * return : struct specinfo or NULL
  */
-STATIC struct specinfo *searchDevByDevnum(
+static struct specinfo *searchDevByDevnum(
 	const int i_devNum)
 {
 	struct specinfo *dev;
-	AggregateList *aggregate = &devSpecHead.aggregate;
 	IteratorList iterator;
 
 	enter_spinlock(&devSpecHead.lock);
 	{
-		aggregate->iterator(aggregate, &iterator);
+		IteratorListConstruct(&iterator, &devSpecHead.aggregate, &devSpecHead.aggrMethod);
 		for (; ; ) {
 			if (iterator.hasNext(&iterator) == BOOL_FALSE) {
 				dev = NULL;
@@ -117,7 +111,7 @@ STATIC struct specinfo *searchDevByDevnum(
  * ǥХϿ
  * return : struct specinfo or NULL
  */
-STATIC struct specinfo *registDev(
+static struct specinfo *registDev(
 	struct cdevsw *i_cdevsw,
 	const size_t i_blockBeg,
 	const size_t i_blocks,
@@ -127,7 +121,6 @@ STATIC struct specinfo *registDev(
 	const int i_perms,
 	const char *i_name)
 {
-	AggregateList *aggregate = &devSpecHead.aggregate;
 	struct specinfo *dev;
 
 	dev = kmalloc(sizeof(*dev));
@@ -164,7 +157,7 @@ STATIC struct specinfo *registDev(
 	listConstructor(&dev->list, dev);
 	enter_spinlock(&devSpecHead.lock);
 	{
-		aggregate->insertHead(aggregate, &dev->list);
+		devSpecHead.aggrMethod.insertHead(&devSpecHead.aggregate, &dev->list);
 	}
 	exit_spinlock(&devSpecHead.lock);
 
@@ -174,14 +167,12 @@ STATIC struct specinfo *registDev(
 /*
  * ǥХϿ
  */
-STATIC void unregistDev(
+static void unregistDev(
 	struct specinfo *m_dev)
 {
-	AggregateList *aggregate = &devSpecHead.aggregate;
-	
 	enter_spinlock(&devSpecHead.lock);
 	{
-		aggregate->removeEntry(aggregate, &m_dev->list);
+		devSpecHead.aggrMethod.removeEntry(&devSpecHead.aggregate, &m_dev->list);
 	}
 	exit_spinlock(&devSpecHead.lock);
 
@@ -191,7 +182,7 @@ STATIC void unregistDev(
 /*
  * cdevsw¤ΤNULLؿѴ롣
  */
-STATIC void compile_devsw(
+static void compile_devsw(
 	struct cdevsw *devsw)
 {
 	if (devsw->d_open == NULL) {
@@ -236,7 +227,7 @@ STATIC void compile_devsw(
 static void (*intrHandler[IRQ_ENTRY])(void *);
 static void *intrArg[IRQ_ENTRY];
 
-STATIC int deviceIntr(int irq)
+static int deviceIntr(int irq)
 {
 	intrHandler[irq](intrArg[irq]);
 
@@ -251,7 +242,7 @@ STATIC int deviceIntr(int irq)
  * ǥХڥ졼˥СȤ롣
  * return : error number
  */
-STATIC int convReadDev(
+static int convReadDev(
 	struct specinfo *i_dev,
 	void *buf,
 	const size_t size,	// READ֥å
@@ -288,7 +279,7 @@ STATIC int convReadDev(
  * ǥХڥ졼˥СȤ롣
  * return : error number
  */
-STATIC int convWriteDev(
+static int convWriteDev(
 	struct specinfo *dev,
 	void *buf,
 	const size_t size,	// WRITE֥å
@@ -321,6 +312,255 @@ STATIC int convWriteDev(
 	return error;
 }
 
+/*
+ * ľܥ֥åǥХIO
+ * return : error number
+ */
+static int ioDirect(
+	void *i_dev,		// ǥХ
+	void *buf,
+	size_t i_blocks,
+	size_t begin,
+	int (*moduleIo)(void*, void*, const size_t, const off_t, void*),
+	size_t *o_blocks)	// READѥ֥å
+{
+	size_t reblocks = i_blocks;
+
+	while (0 < reblocks) {
+		size_t blocks = (reblocks <= MAXPHYS / DEV_BSIZE) ? reblocks : MAXPHYS / DEV_BSIZE;
+		int rest = moduleIo(
+			i_dev,
+			buf,
+			blocks * DEV_BSIZE,
+			begin,
+			NULL);
+		if (rest < 0) {
+			return rest;
+		}
+		else if (0 < rest) {
+			*o_blocks = reblocks - ROUNDUP_DIV(rest, DEV_BSIZE);
+			return NOERR;
+		}
+
+		reblocks -= blocks;
+	}
+	
+	*o_blocks = i_blocks;
+	return NOERR;
+}
+
+/*
+ * ˥᥸㡼ֹ롣
+ * return : ᥸㡼ֹ
+ */
+static int getNewMajor()
+{
+	static int majorNum = 0;
+
+	return majorNum++;
+}
+
+/*
+ * ֥åǥХread
+ * return : IO Хȿ or error number
+ */
+static int readBlk(
+	struct specinfo *dev,
+	void *m_buf,				// Хåե
+	const uint i_bytes,			// Хȿ
+	const uint i_offset)		// ϥХȥեå
+{
+	const uint blkSize = dev->sectSize;
+	char *destBuf = m_buf;
+	uint resid = i_bytes;
+	uint offset = i_offset;
+	char *blkBuf = NULL;
+	uint copyBytes;
+	struct iovec iovec;
+	struct uio uio;
+	int error;
+
+	// ǽΥ֥å̤ɤ
+	if (offset % blkSize) {
+		uint exclBytes = offset % blkSize;
+
+		blkBuf = kmalloc(blkSize);
+		if (blkBuf == NULL) {
+			return -ENOMEM;
+		}
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_READ, ROUNDDOWN(offset, blkSize), blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_read(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+		
+		copyBytes = blkSize - exclBytes;
+		if (resid < copyBytes) {
+			copyBytes = resid;
+		}
+		memcpy(destBuf, blkBuf + exclBytes, copyBytes);
+		destBuf += copyBytes;
+		resid -= copyBytes;
+		offset += copyBytes;
+	}
+
+	// ǸΥ֥åޤɤ
+	while (blkSize <= resid) {
+		copyBytes = (MAXPHYS <= resid) ? MAXPHYS : ROUNDDOWN(resid, blkSize);
+		iovec.iov_base = destBuf;
+		iovec.iov_len = copyBytes;
+		setUio(1, UIO_READ, offset, copyBytes, &iovec, &uio);
+		error = dev->si_devsw->d_read(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+		resid -= copyBytes;
+		if (0 < uio.uio_resid) {
+			resid += uio.uio_resid;
+			goto END;
+		}
+		destBuf += copyBytes;
+		offset += copyBytes;
+	}
+		
+	// ǸΥ֥å̤񤭹
+	if (0 < resid) {
+		if (blkBuf == NULL) {
+			blkBuf = kmalloc(blkSize);
+			if (blkBuf == NULL) {
+				return -ENOMEM;
+			}
+		}
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_READ, offset, blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_read(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+		memcpy(destBuf, blkBuf, resid);
+		resid = 0;
+	}
+END:
+	if (blkBuf != NULL) {
+		kfree(blkBuf);
+	}
+	return i_bytes - resid;
+ERR:
+	if (blkBuf != NULL) {
+		kfree(blkBuf);
+	}
+	return error;
+}
+
+/*
+ * ֥åǥХwrite
+ * return : IO Хȿ or error number
+ */
+static int writeBlk(
+	struct specinfo *dev,
+	void *m_buf,				// Хåե
+	const uint i_bytes,			// Хȿ
+	const uint i_offset)		// ϥХȥեå
+{
+	const uint blkSize = dev->sectSize;
+	char *destBuf = m_buf;
+	uint resid = i_bytes;
+	uint offset = i_offset;
+	char *blkBuf = NULL;
+	uint copyBytes;
+	struct iovec iovec;
+	struct uio uio;
+	int error;
+
+	// ǽΥ֥å̤ɤ
+	if (offset % blkSize) {
+		uint exclBytes = offset % blkSize;
+
+		blkBuf = kmalloc(blkSize);
+		if (blkBuf == NULL) {
+			return -ENOMEM;
+		}
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_READ, ROUNDDOWN(offset, blkSize), blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_read(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+
+		copyBytes = blkSize - exclBytes;
+		if (resid < copyBytes) {
+			copyBytes = resid;
+		}
+		memcpy(blkBuf + exclBytes, destBuf, copyBytes);
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_WRITE, ROUNDDOWN(offset, blkSize), blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_write(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+
+		destBuf += copyBytes;
+		resid -= copyBytes;
+		offset += copyBytes;
+	}
+
+	// ǸΥ֥åޤɤ
+	while (blkSize <= resid) {
+		copyBytes = (MAXPHYS <= resid) ? MAXPHYS : ROUNDDOWN(resid, blkSize);
+		iovec.iov_base = destBuf;
+		iovec.iov_len = copyBytes;
+		setUio(1, UIO_WRITE, offset, copyBytes, &iovec, &uio);
+		error = dev->si_devsw->d_write(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+		resid -= copyBytes;
+		destBuf += copyBytes;
+		offset += copyBytes;
+	}
+		
+	// ǸΥ֥å̤񤭹
+	if (0 < resid) {
+		if (blkBuf == NULL) {
+			blkBuf = kmalloc(blkSize);
+			if (blkBuf == NULL) {
+				return -ENOMEM;
+			}
+		}
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_READ, offset, blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_read(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+
+		memcpy(blkBuf, destBuf, resid);
+		iovec.iov_base = blkBuf;
+		iovec.iov_len = blkSize;
+		setUio(1, UIO_WRITE, offset, blkSize, &iovec, &uio);
+		error = dev->si_devsw->d_write(dev, &uio, 0);
+		if (error != NOERR) {
+			goto ERR;
+		}
+	}
+
+	if (blkBuf != NULL) {
+		kfree(blkBuf);
+	}
+	return i_bytes;
+ERR:
+	if (blkBuf != NULL) {
+		kfree(blkBuf);
+	}
+	return error;
+}
+
 //================================== PUBLIC =============================================
 
 /*
@@ -329,7 +569,7 @@ STATIC int convWriteDev(
  */
 int initDevice()
 {
-	AggregateListConstructor(&devSpecHead.aggregate);
+	AggregateListConstructor(&devSpecHead.aggregate, &devSpecHead.aggrMethod);
 
 	return NOERR;
 }
@@ -390,36 +630,6 @@ struct specinfo *makeBlkDev(
 		m_cdevsw->d_name);
 }
 
-/*
- * ֥åѡƥϿ
- * return : struct specinfo or NULL
- */
-struct specinfo *makeBlkPartDev(
-	struct cdevsw *i_cdevsw,
-	const size_t i_blockBeg,	// ϥ֥åֹ
-	const size_t i_blocks,		// ֥å
-	const int i_partType,		// ѡƥ󥿥
-	const int i_major,
-	const int i_partNum,		// ѡƥֹ
-	const int i_perms)
-{
-	char name[MAX_DEVICE_NAME + 1];
-
-	// ǥХ̾
-	snprintf(name, MAX_DEVICE_NAME + 1, "%s%d", i_cdevsw->d_name, i_partNum);
-	name[MAX_DEVICE_NAME] = '\0';
-
-	return registDev(
-		i_cdevsw,
-		i_blockBeg,
-		i_blocks,
-		i_partType,
-		i_major,
-		i_partNum,
-		i_perms,
-		name);
-}
-
 /*
  * Delete from regist table
  * return : error number
@@ -450,9 +660,9 @@ struct specinfo *getDevByDevnum(
  */
 void setUio(
 	const int iovecCnt,		// struct iovec
-	enum uio_rw uioMode,	// UIO_READ or UIO_WRITE
-	const uint offset,		// Ƭեå
-	const int resid,		// IO
+	enum uio_rw uioRw,		// UIO_READ or UIO_WRITE
+	const off_t offset,		// Ƭեå
+	const int resid,		// 
 	struct iovec *iovec,
 	struct uio *m_uio)		// ꤹstruct uio
 {
@@ -460,8 +670,9 @@ void setUio(
 	m_uio->uio_iovcnt = iovecCnt;
 	m_uio->uio_offset = offset;
 	m_uio->uio_resid = resid;
-	m_uio->uio_segflg = UIO_SYSSPACE;
-	m_uio->uio_rw = uioMode;
+//	m_uio->uio_segflg = UIO_SYSSPACE;
+	m_uio->uio_segflg = UIO_USERSPACE;
+	m_uio->uio_rw = uioRw;
 //	m_uio->uio_procp = NULL;
 }
 
@@ -575,80 +786,6 @@ void setupIntr(
 	intrArg[irq->r_start] = softc;
 }
 
-//----------------------------------------------------------
-// ǥХԤ
-//----------------------------------------------------------
-
-/*
- * ϥɥ
- */
-STATIC void handleDevWake(
-	void *i_task)
-{
-	addToSchedule(i_task, TASK_DEVICE_WAIT);
-}
-
-/*
- *ɬȯθȤʤ륳ޥɤ˸ƤӽФ
- */
-void devInitWait(WAIT_INTR *wait)
-{
-	wait->task = getCurrentTask();
-	wait->isIntr = NO;
-	wait->lock = 0;
-}
-
-/*
- * ǥХԤ
- *WAIT_INTRdevInitWait()ǽƤ뤳
- *        ߤƱCPUȯ뤳
- */
-void devWaitIntr(WAIT_INTR *wait, uint time)
-{
-	void *timer;
-	int error = setTaskTimer(time, 0, handleDevWake, getCurrentTask(), &timer);
-	int eflag;
-
-	eflag = enterCli();
-	{
-		enter_spinlock(&wait->lock);
-
-		if (wait->isIntr == NO) {
-			delFromSchedule(TASK_DEVICE_WAIT);
-			exit_spinlock(&wait->lock);
-			wait_task();
-		}
-		else {
-			exit_spinlock(&wait->lock);
-		}
-	}
-	exitCli(eflag);
-
-	if (error == NOERR){
-		freeTimer(timer);
-	}
-}
-
-/*
- *Գ
- * return : YES = å
- */
-int devWakeIntr(WAIT_INTR *wait)
-{
-	int eflag;
-
-	eflag = enterCli();
-	enter_spinlock(&wait->lock);
-	{
-		wait->isIntr = YES;
-		addScheduleSoon(wait->task, TASK_DEVICE_WAIT);
-	}
-	exit_spinlock(&wait->lock);
-	exitCli(eflag);
-
-	return YES;
-}
-
 //----------------------------------------------------------
 // ľžؿ
 //----------------------------------------------------------
@@ -659,25 +796,11 @@ int devWakeIntr(WAIT_INTR *wait)
 int read_direct(
 	void *i_dev,		// ǥХ
 	void *buf,
-	size_t i_sectors,
+	size_t i_blocks,
 	size_t begin,
-	size_t *o_sectors)	// READѥ֥å
+	size_t *o_blocks)	// readѥ֥å
 {
-	struct specinfo *dev = i_dev;
-	size_t sectors;
-
-	/* ǥ¤Υå */
-	if (dev->sectors < begin){
-		return -EIO;
-	}
-	if ((begin + i_sectors) <= dev->sectors){
-		sectors = i_sectors;
-	}
-	else{
-		sectors = dev->sectors - begin;
-	}
-
-	return convReadDev(dev, buf, sectors, begin + dev->beginSector, 0, o_sectors);
+	return ioDirect(i_dev, buf, i_blocks, begin, moduleReadDev, o_blocks);
 }
 
 /*
@@ -686,25 +809,11 @@ int read_direct(
 int write_direct(
 	void *i_dev,		// ǥХ
 	void *buf,
-	size_t i_sectors,
+	size_t i_blocks,
 	size_t begin,
-	size_t *o_sectors)	// WRITEѥ֥å
+	size_t *o_blocks)	// writeѥ֥å
 {
-	struct specinfo *dev = i_dev;
-	size_t sectors;
-
-	/* ǥ¤Υå */
-	if (dev->sectors < begin){
-		return -EIO;
-	}
-	if ((begin + i_sectors) <= dev->sectors){
-		sectors = i_sectors;
-	}
-	else{
-		sectors = dev->sectors - begin;
-	}
-
-	return convWriteDev(dev, buf, sectors, begin + dev->beginSector, 0, o_sectors);
+	return ioDirect(i_dev, buf, i_blocks, begin, moduleWriteDev, o_blocks);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -740,6 +849,7 @@ int devCloseDev(
 
 /*
  * ǥХɤ߹
+ *ԳԲġ
  * return : read bytes or error number
  */
 int devReadDev(
@@ -764,66 +874,12 @@ int devReadDev(
 			return ioSize;
 		}
 	// ֥åǥХ
-	case S_IFBLK:{
-		char *sectBuf = NULL;
-		int sectBeg = i_offset / i_dev->sectSize;	// žϤΥֹ
-		size_t restBytes = i_bytes;					// ĤžХȿ
-		size_t sectors;
-
-		/* ǽΥ̤ž */
-		if (0 < (i_offset % i_dev->sectSize)){
-			int forward = i_offset % i_dev->sectSize;
-			int back;
-
-			sectBuf = kmalloc(i_dev->sectSize);
-			if (sectBuf == NULL){
-				return -ENOMEM;
-			}
-			error = read_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			back = i_dev->sectSize - forward;
-			if (i_bytes < back){
-				back = i_bytes;
-			}
-			memcpy(readBuf, sectBuf + forward, back);
-			readBuf += back;
-			sectBeg += 1;
-			restBytes -= back;
-		}
-
-		sectors = restBytes / i_dev->sectSize;
-		error = read_cache(i_dev, readBuf, sectors, sectBeg);
-		if (error != NOERR){
-			goto ERR;
-		}
-		restBytes -= sectors * i_dev->sectSize;
-		sectBeg += sectors;
-		readBuf += sectors * i_dev->sectSize;
-
-		/* Ĥ꤬̤ž */
-		if (0 < restBytes){
-			if (sectBuf == NULL){
-				sectBuf = kmalloc(i_dev->sectSize);
-				if (sectBuf == NULL){
-					return -ENOMEM;
-				}
-			}
-			error = read_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			memcpy(readBuf, sectBuf, restBytes);
-			restBytes = 0;
-		}
-
-		kfree(sectBuf);
-		return i_bytes - restBytes;
-ERR:
-		kfree(sectBuf);
-		return error;
-	}
+	case S_IFBLK:
+/****************************************************************************************************************/
+printk("readBlk m_buf=%x bytes=%d offset=%x PageDir=%x\n", 
+	m_buf, i_bytes, i_offset, getPageDir(getVmTask(getCurrentTask())));
+/****************************************************************************************************************/
+		return readBlk(i_dev, m_buf, i_bytes, i_offset);
 	// root directory
 	case S_IFDIR:
 		return 0;
@@ -835,6 +891,7 @@ ERR:
 
 /*
  * ǥХ˽񤭹
+ *ԳԲġ
  * return : write bytes or error number
  */
 int devWriteDev(
@@ -859,75 +916,9 @@ int devWriteDev(
 			return ioSize;
 		}
 	// ֥åǥХ
-	case S_IFBLK:{
-		char *sectBuf = NULL;
-		int sectBeg = i_offset / i_dev->sectSize;	// žϤΥֹ
-		size_t restSize = i_bytes;					// ĤžХȿ
-		size_t sectors;
-
-		/* ǽΥ̤ž */
-		if (0 < (i_offset % i_dev->sectSize)){
-			int forward = i_offset % i_dev->sectSize;
-			int back;
-
-			sectBuf = kmalloc(i_dev->sectSize);
-			if (sectBuf == NULL){
-				return -ENOMEM;
-			}
-			error = read_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			back = i_dev->sectSize - forward;
-			if (i_bytes < back){
-				back = i_bytes;
-			}
-			memcpy(sectBuf + forward, writeBuf, back);
-			error = write_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			writeBuf += back;
-			sectBeg += 1;
-			restSize -= back;
-		}
-
-		sectors = restSize / i_dev->sectSize;
-		error = write_cache(i_dev, writeBuf, sectors, sectBeg);
-		if (error != NOERR){
-			goto ERR;
-		}
-		restSize -= sectors * i_dev->sectSize;
-		sectBeg += sectors;
-		writeBuf += sectors * i_dev->sectSize;
-
-		/* Ĥ꤬̤ž */
-		if (0 < restSize){
-			if (sectBuf == NULL){
-				sectBuf = kmalloc(i_dev->sectSize);
-				if (sectBuf == NULL){
-					return -ENOMEM;
-				}
-			}
-			error = read_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			memcpy(sectBuf, writeBuf, restSize);
-			error = write_cache(i_dev, sectBuf, 1, sectBeg);
-			if (error != NOERR){
-				goto ERR;
-			}
-			restSize = 0;
-		}
-
-		kfree(sectBuf);
-		return i_bytes - restSize;
-ERR:
-		kfree(sectBuf);
-		return error;
-	}
-	/* root directory */
+	case S_IFBLK:
+		return writeBlk(i_dev, i_buf, i_bytes, i_offset);
+	// root directory
 	case S_IFDIR:
 		return 0;
 	default:
@@ -981,7 +972,46 @@ void devStatDev(
 	m_devStat->name			= i_dev->si_name;
 }
 
-/****************************************************************************/
+void statDev(
+	void *i_dev,
+	DEV_STAT *m_devStat)
+{
+	if (((struct specinfo*) i_dev)->moduleFlag == 0) {
+		devStatDev(i_dev, m_devStat);
+	}
+	else {
+		moduleStatDev(i_dev, m_devStat);
+	}
+}
+
+int major(
+	const dev_t x)
+{
+	if (x == NODEV) {
+		return NOUDEV;
+	}
+
+	return getMajor(x->si_udev);
+}
+
+int minor(
+	const dev_t x)
+{
+	if (x == NODEV) {
+		return NOUDEV;
+	}
+
+	return getMinor(x->si_udev);
+}
+
+udev_t makeudev(
+	int x, 
+	int y)
+{
+	return ((x << 8) | y);
+}
+
+//*************************************************************************************************
 void test_device()
 {
 }
