@@ -8,7 +8,6 @@
  * ե륷ƥ
  */
 
-
 #include <sys/config.h>
 #include <sys/types.h>
 #include <sys/param.h>
@@ -22,18 +21,18 @@
 #include <lib/lib_path.h>
 #include <kern/kmalloc.h>
 #include <kern/proc.h>
-#include <kern/Thread.h>
-#include <kern/errno.h>
+#include <sys/Thread.h>
+#include <kern/TaskWait.h>
 #include <kern/time.h>
 #include <kern/ProcSignal.h>
 #include <kern/devfs.h>
+#include <kern/FsRegist.h>
 #include <kern/vfs.h>
 #include <kern/fs.h>
 #include <kern/timer.h>
 
 #include <kern/debug.h>
 
-
 //#define DEBUG_FS 1
 #ifdef DEBUG_FS
 	#define STATIC
@@ -43,7 +42,6 @@
 	#define INLINE	inline
 #endif
 
-
 //================================== PRIVATE ============================================
 
 /*
@@ -75,14 +73,14 @@ STATIC int getEmptyFdNum(FILE_STRUCT *fstruct, const uint min)
 
 /*
  *աղۥȥ꡼ޥȥȥ꡼Ǥ뤳ȡ
- * Set current directory.
+ *ԳԲġ
  */
 STATIC void setCurrentDir(void *vent)
 {
 	FILE_STRUCT *fstruct = getFileStructProc(getCurrentProc());
 
 	if (fstruct->current_dir != NULL) {
-		closeVent(fstruct->current_dir);
+		VfsClose(fstruct->current_dir);
 	}
 
 	fstruct->current_dir = vent;
@@ -179,15 +177,15 @@ STATIC int setOpenFlag(uint num,int flag)
  * Copy file descriptor.
  * return : file descriptor number or error number
  */
-STATIC int copyFd(int num1,int num2)
+STATIC int copyFd(int num1, int num2)
 {
 	FILE_STRUCT *fstruct = getFileStructProc(getCurrentProc());
 
 	// ǥץֹ
-	if (_POSIX_OPEN_MAX <= (uint)num2) {
+	if (_POSIX_OPEN_MAX <= (uint) num2) {
 		num2 = 0;
 	}
-	num2 = getEmptyFdNum(fstruct,num2);
+	num2 = getEmptyFdNum(fstruct, num2);
 
 	// եǥץ򥳥ԡ
 	fstruct->fd[num2] = fstruct->fd[num1];
@@ -201,6 +199,7 @@ STATIC int copyFd(int num1,int num2)
 
 /*
  * file descriptorλȥ󥿤򸺤餹
+ *ԳԲġ
  * return : error number
  */
 STATIC int unlinkFd(FILE_STRUCT *fstruct, int fdNum)
@@ -215,7 +214,7 @@ STATIC int unlinkFd(FILE_STRUCT *fstruct, int fdNum)
 		}
 	}
 	else{
-		closeVent(fd->vent);
+		VfsClose(fd->vent);
 		if (fd->refCount <= 0){
 			kfree(fd);
 		}
@@ -261,6 +260,7 @@ FILE_STRUCT *copyFileStruct(FILE_STRUCT *fstruct)
 
 /*
  * եطι¤Τ롣
+ *ԳԲġ
  */
 void releaseFileStruct(FILE_STRUCT *fstruct)
 {
@@ -280,6 +280,7 @@ void releaseFileStruct(FILE_STRUCT *fstruct)
 
 /*
  * ե빽¤Τν
+ *ԳԲġ
  */
 void initExecFs()
 {
@@ -437,9 +438,12 @@ STATIC int getFdFromFdset(const int lastFdNum, const int firstFd, const fd_set *
 
 /*
  * ¹ԥե륪ץ̾Υץδάؿ
+ *ԳԲġ
  * return : error number
  */
-int exec_open(const char *i_path, EXEC_FD *fd)
+int exec_open(
+	const char *i_path, 
+	EXEC_FD *fd)
 {
 	void *vent;
 	int error;
@@ -451,7 +455,7 @@ int exec_open(const char *i_path, EXEC_FD *fd)
 		return -ENAMETOOLONG;
 	}
 
-	error = openObj(i_path, O_RDONLY, &vent);
+	error = getVent(i_path, &vent);
 	if (error != NOERR) {
 		return error;
 	}
@@ -465,7 +469,7 @@ int exec_open(const char *i_path, EXEC_FD *fd)
 
 /*
  * ¹ԥեɤ߹ߡ"sys_read"δάؿ
- * parameters : open file struct,buffer,read size
+ *ԳԲġ
  * return : read bytes or error number
  */
 int exec_read(EXEC_FD *fd, void *buf, size_t size)
@@ -475,13 +479,17 @@ int exec_read(EXEC_FD *fd, void *buf, size_t size)
 
 /*
  * ¹ԥեΥ
+ *ԳԲġ
  */
 void exec_close(EXEC_FD *fd)
 {
 	releaseVent(fd->vent);
 }
 
-int sys_mount(const char *devPath, const char *fsName, const char *i_path)
+int sys_mount(
+	const char *devPath, 
+	const char *fsName, 
+	const char *i_path)
 {
 	void *mountRoot;
 	void *mountDevDsc;
@@ -489,9 +497,6 @@ int sys_mount(const char *devPath, const char *fsName, const char *i_path)
 	void *vent;
 	int error;
 
-	if (PATH_MAX < strnlen(devPath, PATH_MAX + 1)){
-		return -ENAMETOOLONG;
-	}
 	if (PATH_MAX < strnlen(i_path, PATH_MAX + 1)){
 		return -ENAMETOOLONG;
 	}
@@ -502,15 +507,24 @@ int sys_mount(const char *devPath, const char *fsName, const char *i_path)
 		return -ENOENT;
 	}
 
-	/* ǥХץ */
-	error = openDevice(devPath, &mountDevDsc);
-	if (error < 0){
-		return error;
+	if (devPath != NULL) {
+		if (PATH_MAX < strnlen(devPath, PATH_MAX + 1)){
+			return -ENAMETOOLONG;
+		}
+
+		/* ǥХץ */
+		error = devOpen(devPath, &mountDevDsc);
+		if (error != NOERR){
+			return error;
+		}
+	}
+	else {
+		mountDevDsc = NULL;
 	}
 
 	// ޥȴؿƤӽФ
 	error = mountFs->mount(mountDevDsc, &mountRoot);
-	if (error < 0){
+	if (error != NOERR) {
 		return error;
 	}
 
@@ -521,13 +535,13 @@ int sys_mount(const char *devPath, const char *fsName, const char *i_path)
 
 	// ۥǥ쥯ȥޥȤ
 	error = mountToVfs(mountFs, mountDevDsc, mountRoot, vent);
-	if (error != NOERR){
+	if (error != NOERR) {
 		goto ERR2;
 	}
 
 	return NOERR;
 ERR2:
-	closeVent(vent);
+	VfsClose(vent);
 ERR:
 	mountFs->umount(mountDevDsc, mountRoot);
 	return error;
@@ -681,7 +695,7 @@ int sys_open(const char *i_path, const int i_oflag, const mode_t i_mode)
 
 	return fdNum;
 ERR2:
-	closeVent(vent);
+	VfsClose(vent);
 ERR:
 	return error;
 }
@@ -705,7 +719,7 @@ int sys_read(int fdNum, void *buf, size_t size)
 	int rest;
 
 	/* Хåեå */
-	if (checkMem(buf, size) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), buf, size) == ERR) {
 		return -EFAULT;
 	}
 
@@ -738,17 +752,12 @@ int sys_write(int fdNum, void *buf, size_t size)
 	int rest;
 
 	/* Хåեå */
-	if (checkMem(buf, size) == ERR){
+	if (vmIsReadArea(getVmProc(getCurrentProc()), buf, size) == ERR){
 		return -EFAULT;
 	}
 
 	fd = getFd(fdNum);
 	if (fd == NULL){
-/****************************************************************************************************************/
-((char*) buf)[size] = '\0';
-printk("%s", buf);
-return size;
-/****************************************************************************************************************/
 		return -EBADF;
 	}
 
@@ -758,7 +767,7 @@ return size;
 	}
 
 	if (getTypeVent(fd->vent) == S_IFIFO) {
-		rest = readVent(fd->vent, size, 0, buf);
+		rest = writeVent(fd->vent, size, 0, buf);
 	}
 	else{
 		rest = writeVent(fd->vent, size, fd->f_pos, buf);
@@ -855,7 +864,7 @@ int sys_opendir(const char *i_path)
 
 	return fdNum;
 ERR2:
-	closeVent(vent);
+	VfsClose(vent);
 ERR:
 	return error;
 }
@@ -864,7 +873,7 @@ int sys_readdir(int dirNum, char *name)
 {
 	DIR_STRAEM *dir;
 
-	dir = (DIR_STRAEM*)getFd(dirNum);
+	dir = (DIR_STRAEM*) getFd(dirNum);
 	if (dir == NULL){
 		return -EBADF;
 	}
@@ -953,7 +962,7 @@ int sys_getcwd(char *strBuf, size_t size)
 	}
 
 	/* Хåեå */
-	if (checkMem(strBuf, size) == ERR){
+	if (vmIsReadArea(getVmProc(getCurrentProc()), strBuf, size) == ERR){
 		return -EFAULT;
 	}
 
@@ -969,7 +978,7 @@ int sys_getcwd(char *strBuf, size_t size)
 
 int sys_stat(const char *i_path, struct stat *m_stat)
 {
-	if (checkMem(m_stat, sizeof(struct stat)) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), m_stat, sizeof(struct stat)) == ERR) {
 		return -EFAULT;
 	}
 	if (PATH_MAX < strnlen(i_path, PATH_MAX + 1)) {
@@ -983,10 +992,16 @@ int sys_fstat(int fdNum, struct stat *m_stat)
 {
 	F_DSC *fd;
 
-	if (checkMem(m_stat, sizeof(struct stat)) == ERR){
+	if (vmIsReadArea(getVmProc(getCurrentProc()), m_stat, sizeof(struct stat)) == ERR){
+/****************************************************************************************************************/
+printk("sys_fstat EFAULT\n");
+/****************************************************************************************************************/
 		return -EFAULT;
 	}
 	if ((fd = getFd(fdNum)) == NULL){
+/****************************************************************************************************************/
+printk("sys_fstat EBADF\n");
+/****************************************************************************************************************/
 		return -EBADF;
 	}
 	m_stat->st_crtpos = fd->f_pos;
@@ -996,7 +1011,7 @@ int sys_fstat(int fdNum, struct stat *m_stat)
 
 int sys_lstat(const char *i_path, struct stat *m_stat)
 {
-	if (checkMem(m_stat, sizeof(struct stat)) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), m_stat, sizeof(struct stat)) == ERR) {
 		return -EFAULT;
 	}
 	if (PATH_MAX < strnlen(i_path, PATH_MAX + 1)) {
@@ -1019,14 +1034,14 @@ int sys_umount_root()
 int sys_fcntl(int fdNum, int cmd, uint arg)
 {
 	switch(cmd){
-	case F_DUPFD:{
+	case F_DUPFD: {
 		if (isFileOpenMax() == YES){
 			return -EMFILE;
 		}
 		if (getFd(fdNum) == NULL){			/* File descriptorΥå */
 			return -EBADF;
 		}
-		return copyFd(fdNum,arg);			/* Copy File descriptor. */
+		return copyFd(fdNum, arg);			/* Copy File descriptor. */
 	}
 	case F_GETFD:
 		return getFdFlag(fdNum);
@@ -1136,14 +1151,19 @@ int sys_readlink(const char *i_path, char *readBuf, size_t bufSize)
 	if (PATH_MAX < strnlen(i_path, PATH_MAX + 1)){
 		return -ENAMETOOLONG;
 	}
-	if (checkMem(readBuf, bufSize) == ERR){
+	if (vmIsReadArea(getVmProc(getCurrentProc()), readBuf, bufSize) == ERR){
 		return -EFAULT;
 	}
 
 	return readlinkPath(i_path, bufSize, readBuf);
 }
 
-int sys_select(const int lastFdNum, fd_set *m_fdsRead, fd_set *m_fdsWrite, fd_set *m_fdsError, struct timeval *timeout)
+int sys_select(
+	const int lastFdNum, 
+	fd_set *m_fdsRead, 
+	fd_set *m_fdsWrite, 
+	fd_set *m_fdsError, 
+	struct timeval *timeout)
 {
 	enum{
 		FDS_LAST = 3,
@@ -1174,7 +1194,7 @@ int sys_select(const int lastFdNum, fd_set *m_fdsRead, fd_set *m_fdsWrite, fd_se
 	isFds = NO;
 	for (i = 0; i < FDS_LAST; ++i){
 		if (fdSet[i] != NULL){
-			if (checkMem(fdSet[i], sizeof(fd_set)) == ERR){
+			if (vmIsReadArea(getVmProc(getCurrentProc()), fdSet[i], sizeof(fd_set)) == ERR){
 				return -EFAULT;
 			}
 			isFds = YES;
@@ -1186,6 +1206,7 @@ int sys_select(const int lastFdNum, fd_set *m_fdsRead, fd_set *m_fdsWrite, fd_se
 		// ٥Ȥθ
 		for (;;){
 			int fdNum;
+			uint remainTime;
 
 			for (fdNum = 0; (fdNum = getFdFromFdset(lastFdNum, fdNum, &fdSetAll)) != -1;){
 				int c_update = 0;
@@ -1203,7 +1224,7 @@ int sys_select(const int lastFdNum, fd_set *m_fdsRead, fd_set *m_fdsWrite, fd_se
 					}
 				}
 				rest = pollVent(fd->vent, events);
-				
+
 				// ǽFDΥ
 				for (i = 0; i < FDS_LAST; ++i){
 					if ((rest & pollEvents[i]) == pollEvents[i]){
@@ -1245,20 +1266,19 @@ int sys_select(const int lastFdNum, fd_set *m_fdsRead, fd_set *m_fdsWrite, fd_se
 			}
 			
 			// Ԥ
-			waitTimeSignal(miliSeconds);
+			remainTime = TaskWaitTimer(miliSeconds);
 
 			// ʥߤ
 			if(isSigint(TaskGetTaskSignal(getCurrentTask())) == YES) {
 				return -EINTR;
 			}
 
-			// ܤԤ֤򥯥ꥢ
-			miliSeconds = 0;
+			miliSeconds = remainTime;
 		}
 	}
 	else{
 		// ʥԤ
-		waitTimeSignal(miliSeconds);
+		TaskWaitTimer(miliSeconds);
 
 		return 0;
 	}
@@ -1277,7 +1297,7 @@ int sys_poll(
 	}
 
 	// ꡼å
-	if (checkMem(m_pollfd, sizeof(*m_pollfd) * i_nfd) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), m_pollfd, sizeof(*m_pollfd) * i_nfd) == ERR) {
 		return -EFAULT;
 	}
 	
@@ -1292,6 +1312,7 @@ int sys_poll(
 	// ٥Ȥθ
 	for (;;){
 		int isWait = NO;
+		uint remainTime;
 		int rest;
 		int i;
 
@@ -1346,12 +1367,14 @@ int sys_poll(
 		if (miliSeconds == 0){
 			break;
 		}
-		waitTimeSignal(miliSeconds);
+		remainTime = TaskWaitTimer(miliSeconds);
 
 		// ʥߤ
 		if(isSigint(TaskGetTaskSignal(getCurrentTask())) == YES) {
 			return -EINTR;
 		}
+
+		miliSeconds = remainTime;
 	}
 
 	return c_complete;
@@ -1364,7 +1387,7 @@ int sys_getstatfs(struct statfs *m_statfs, long bufsize, int flags)
 
 	count = (statfs == NULL) ? INT_MAX : bufsize / sizeof(struct statfs);
 	if (statfs != NULL) {
-		if (checkMem(statfs, sizeof(*statfs) * count) == ERR){
+		if (vmIsReadArea(getVmProc(getCurrentProc()), statfs, sizeof(*statfs) * count) == ERR){
 			return -EFAULT;
 		}
 	}
@@ -1380,7 +1403,7 @@ int sys_statfs(const char *i_path, struct statfs *m_statfs)
 	if (PATH_MAX < strnlen(i_path, PATH_MAX + 1)){
 		return -ENAMETOOLONG;
 	}
-	if (checkMem(m_statfs,sizeof(struct statfs)) == ERR){
+	if (vmIsReadArea(getVmProc(getCurrentProc()), m_statfs,sizeof(struct statfs)) == ERR){
 		return -EFAULT;
 	}
 
@@ -1410,57 +1433,9 @@ int sys_lock()
 	return 0;
 }
 
-
-/************************************************************************************
- *
- * 
- *
- ************************************************************************************/
-
-//================================== PUBLIC =============================================
-
-/*
- * Init process0 file struct.
- * return : error number
- */
-int setInitProcFs()
-{
-	int fdNum;
-
-	/*
-	 * ɸϤ򳫤
-	 */
-	fdNum = sys_open("/dev/console", O_RDONLY, 0);	/* ɸϡ */
-	if (fdNum < 0){
-		return fdNum;
-	}
-	fdNum = setFdFlag(fdNum, 0);						/* No exec fd close. */
-	if (fdNum < 0){
-		return fdNum;
-	}
-	fdNum = sys_open("/dev/console", O_WRONLY, 0);	/* ɸϡ */
-	if (fdNum < 0){
-		return fdNum;
-	}
-	fdNum = setFdFlag(fdNum, 0);
-	if (fdNum < 0){
-		return fdNum;
-	}
-	fdNum = sys_open("/dev/console", O_WRONLY, 0);	/* ɸ२顼 */
-	if (fdNum < 0){
-		return fdNum;
-	}
-	fdNum = setFdFlag(fdNum, 0);
-	if (fdNum < 0){
-		return fdNum;
-	}
-
-	return NOERR;
-}
-
 /*************************************************************************************************/
 
 void testFs(int num, int nest)
 {
-	printk("testFs() %d %d\n", num, nest);
+	debugFlag ^= 1;
 }
