@@ -14,16 +14,18 @@
 #include <sys/limits.h>
 #include <sys/times.h>
 #include <sys/wait.h>
-#include <sys/AggregateList.h>
+#include <lib/IteratorList.h>
+#include <lib/AggregateList.h>
 #include <machine/interrupt.h>
+#include <machine/lock.h>
 #include <kern/kmalloc.h>
 #include <kern/time.h>
 #include <kern/term.h>
 #include <kern/Hierarchy.h>
 #include <kern/ProcSignal.h>
 #include <kern/TaskSignal.h>
-#include <kern/Wait.h>
-#include <kern/Thread.h>
+#include <kern/TaskWait.h>
+#include <sys/Thread.h>
 
 #include <kern/debug.h>
 
@@ -44,8 +46,6 @@ enum{
 	// ץ
 	PROC_RUNNING = 1,
 	PROC_EXIT = 2,
-	
-	MAX_CHILD = 8,		/* Max number of child processes */
 };
 
 /*
@@ -53,7 +53,6 @@ enum{
  */
 struct proc {
 	int					state;			// ץξ
-	int					childCount;		// Number of child processes
 	uint				ctime[2];		// ҥץ׻ system = 0,user = 1
 	uchar				exit_state;		// Exit state.
 	uchar				signum;			// Exit signal number.
@@ -70,14 +69,15 @@ struct proc {
 
 	// å
 	AggregateList		aggrThread;		// åɥꥹȽ
+	AggregateListMethod	aggrMethod;		// åɥꥹȥ᥽å
 
 	ITIMER				itimerVirtual;	// itimer
 	ITIMER				itimerProf;		// itimer
 	void				*ctlterm;		// ߥʥ湽¤
 	VmProcObj			vmProcObj;		// ץ꡼¤
 	void				*fileStruct;	// ե빽¤Υɥ쥹
-	int					spinLock;		// ԥå
 	ProcSignalObj		procSignalObj;	// ʥ¤Ρ
+	int					spinLock;		// ԥå
 };
 
 typedef struct proc PROC;
@@ -123,9 +123,8 @@ STATIC void procConstructor(
 {
 	memset(this, 0, sizeof(*this));
 
-	HierarchyConstructor(&this->hierarchy, this);
-	AggregateListConstructor(&this->aggrThread);
-
+	HierarConstructor(&this->hierarchy, this);
+	AggregateListConstructor(&this->aggrThread, &this->aggrMethod);
 	if (i_parent != NULL) {
 		this->pid = getNewPid();
 		this->pgid = i_parent->pgid;
@@ -133,7 +132,6 @@ STATIC void procConstructor(
 		this->gid = i_parent->gid;
 		this->sid = i_parent->sid;
 	}
-
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -150,16 +148,15 @@ STATIC int getThread(
 	pthread_t (*getThreadId)(ThreadObj*),
 	ThreadObj **o_thread)
 {
-	AggregateList *aggrThread = &i_proc->aggrThread;
 	IteratorList iterator;
 	int error = NOERR;
 	int eflags;
 
 	// åɤθ
-	aggrThread->iterator(aggrThread, &iterator);
 	eflags = enterCli();
 	enter_spinlock(&i_proc->spinLock);
 	{
+		IteratorListConstruct(&iterator, &i_proc->aggrThread, &i_proc->aggrMethod);
 		for (;;) {
 			ThreadObj *thread;
 
@@ -196,16 +193,14 @@ STATIC int sendTaskMessage(
 	uint param5,
 	uint param6)
 {
-	AggregateList *aggrThread = &proc->aggrThread;
 	IteratorList iterator;
 	int eflags;
 	int error = NOERR;
 
-	aggrThread->iterator(aggrThread, &iterator);
-
 	eflags = enterCli();
 	enter_spinlock(&proc->spinLock);
 	{
+		IteratorListConstruct(&iterator, &proc->aggrThread, &proc->aggrMethod);
 		while (iterator.hasNext(&iterator) == BOOL_TRUE) {
 			void *thread = iterator.next(&iterator);
 
@@ -230,12 +225,11 @@ STATIC void getProcTime(
 	int *o_sysTime,		// ƥॿ
 	int *o_userTime)	// 桼
 {
-	AggregateList *aggregate = &i_proc->aggrThread;
 	IteratorList iterator;
 	int sysTime = 0;
 	int userTime = 0;
 
-	aggregate->iterator(aggregate, &iterator);
+	IteratorListConstruct(&iterator, &i_proc->aggrThread, &i_proc->aggrMethod);
 	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
 		void *thread = iterator.next(&iterator);
 		sysTime += calcMiliSecondFromClock(getSysClocks(thread)) / TASK_TIME;
@@ -279,6 +273,7 @@ STATIC void setState(
 /*
  * EXITҥץ
  *ոƤӽФ¾򤫤뤳
+ *ԳԲġ
  */
 STATIC void releaseChild(
 	PROC *this)
@@ -287,16 +282,17 @@ STATIC void releaseChild(
 	HierarchyObj *childHierarchy;
 	HierarchyObj *next;
 
-	for (childHierarchy = HierarchyGetFirstChild(hierarchy); childHierarchy != NULL; childHierarchy = next) {
-		next = HierarchyGetNextChild(hierarchy, childHierarchy);
-		PROC *child = HierarchyGetProc(childHierarchy);
+	for (childHierarchy = HierarGetFirstChild(hierarchy); childHierarchy != NULL; childHierarchy = next) {
+		next = HierarGetNextChild(hierarchy, childHierarchy);
+		PROC *child = HierarGetProc(childHierarchy);
 		if (child->state == PROC_EXIT) {
 			// åɤγ
 			AggregateList *aggregate = &child->aggrThread;
+			AggregateListMethod *aggrMethod = &child->aggrMethod;
 			void *thread;
-			while ((thread = aggregate->refHead(aggregate)) != NULL) {
+			while ((thread = aggrMethod->refHead(aggregate)) != NULL) {
 				if (getTaskState(thread) == TASK_EXIT) {
-					freeThread(aggregate->getHead(aggregate), &child->vmProcObj);
+					freeThread(aggrMethod->getHead(aggregate), &child->vmProcObj);
 				}
 				else {
 					wait_task();
@@ -307,7 +303,7 @@ STATIC void releaseChild(
 			ProcSignalDestructor(&child->procSignalObj);
 
 			// ץؤ
-			HierarchyRemoveChild(hierarchy, childHierarchy);
+			HierarRemoveChild(hierarchy, childHierarchy);
 
 			kfree(child);
 		}
@@ -324,11 +320,11 @@ STATIC void addChildProc1(
 	HierarchyObj *childHierarchy;
 	HierarchyObj *next;
 
-	for (childHierarchy = HierarchyGetFirstChild(&this->hierarchy); childHierarchy != NULL; childHierarchy = next) {
-		next = HierarchyGetNextChild(&this->hierarchy, childHierarchy);
+	for (childHierarchy = HierarGetFirstChild(&this->hierarchy); childHierarchy != NULL; childHierarchy = next) {
+		next = HierarGetNextChild(&this->hierarchy, childHierarchy);
 
-		HierarchyRemoveChild(&this->hierarchy, childHierarchy);
-		HierarchyAddChild(&initProc->hierarchy, childHierarchy);
+		HierarRemoveChild(&this->hierarchy, childHierarchy);
+		HierarAddChild(&initProc->hierarchy, childHierarchy);
 	}
 }
 
@@ -354,8 +350,8 @@ STATIC int searchExitState(
 
 	if (i_pid != 0) {
 		hierarchy = &i_parent->hierarchy;
-		for (childHierar = HierarchyGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarchyGetNextChild(hierarchy, childHierar)) {
-			PROC *child = HierarchyGetProc(childHierar);
+		for (childHierar = HierarGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarGetNextChild(hierarchy, childHierar)) {
+			PROC *child = HierarGetProc(childHierar);
 			if (child->pid == i_pid) {
 				rest = 0;
 				if (child->state & i_state) {
@@ -368,8 +364,8 @@ STATIC int searchExitState(
 	}
 	else if (i_pgid != 0) {
 		hierarchy = &i_parent->hierarchy;
-		for (childHierar = HierarchyGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarchyGetNextChild(hierarchy, childHierar)) {
-			PROC *child = HierarchyGetProc(childHierar);
+		for (childHierar = HierarGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarGetNextChild(hierarchy, childHierar)) {
+			PROC *child = HierarGetProc(childHierar);
 			if (child->pgid == i_pgid) {
 				rest = 0;
 				if (child->state & i_state) {
@@ -382,8 +378,8 @@ STATIC int searchExitState(
 	}
 	else {
 		hierarchy = &i_parent->hierarchy;
-		for (childHierar = HierarchyGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarchyGetNextChild(hierarchy, childHierar)) {
-			PROC *child = HierarchyGetProc(childHierar);
+		for (childHierar = HierarGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarGetNextChild(hierarchy, childHierar)) {
+			PROC *child = HierarGetProc(childHierar);
 			rest = 0;
 			if (child->state & i_state) {
 				setState(child, o_state);
@@ -398,6 +394,9 @@ STATIC int searchExitState(
 
 /*
  * ץե
+ *ԳԲġ
+ * return : if parent process then child ID or error number, 
+ *          if child process then 0.
  */
 STATIC int fork(
 	int isKernelThread)		// isKernelThreadˤϥͥ륹åɤȤ"0"ʳ
@@ -405,10 +404,9 @@ STATIC int fork(
 	PROC *this = getCurrentProc();
 	PROC *child;
 	ThreadObj *newThread;
-	AggregateList *parentAggregate;
 	int error;
 
-	if (MAX_CHILD <= this->childCount) {
+	if (CHILD_MAX <= HierarGetChildCount(&this->hierarchy)) {
 		return -EAGAIN;
 	}
 
@@ -419,12 +417,12 @@ STATIC int fork(
 	}
 	procConstructor(child, this);
 
-	// ץؤϿ
-	HierarchyAddChild(&this->hierarchy, &child->hierarchy);
-
 	// å󥢥ɥ쥹(ü¤Υɥ쥹)򥳥ԡ
 	child->ctlterm = this->ctlterm;
 
+	// ץؤϿ
+	HierarAddChild(&this->hierarchy, &child->hierarchy);
+
 	// եط¤Τγ
 	child->fileStruct = copyFileStruct(getFileStructProc(this));
 	if (getFileStructProc(child) == NULL) {
@@ -443,11 +441,10 @@ STATIC int fork(
 	vmProcFork(&child->vmProcObj, &this->vmProcObj);
 
 	// åɤκ
-	parentAggregate = &this->aggrThread;
 	error = forkThread(
 		isKernelThread, 
 		child, 
-		parentAggregate->refEnd(parentAggregate), 
+		this->aggrMethod.refEnd(&this->aggrThread), 
 		&child->vmProcObj, 
 		&newThread);
 	if (error == 1) {
@@ -460,15 +457,17 @@ STATIC int fork(
 	}
 
 	// åɤץϿ
-	registThread(child, getProcList(newThread));
+	error = registThread(child, getProcList(newThread));
+	if (error != NOERR) {
+		goto ERR;
+	}
 
-	// 塼˲ä
+	// ҥץ򥿥塼˲ä
 	addNewTask(newThread);
 
-	++this->childCount;
-
 	return child->pid;
 ERR:
+	HierarRemoveChild(&this->hierarchy, &child->hierarchy);
 	kfree(child);
 
 	return error;
@@ -490,8 +489,8 @@ STATIC void sendSignalPgid(
 	HierarchyObj *hierarchy = &i_parent->hierarchy;
 	HierarchyObj *childHierar;
 
-	for (childHierar = HierarchyGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarchyGetNextChild(hierarchy, childHierar)) {
-		PROC *child = HierarchyGetProc(childHierar);
+	for (childHierar = HierarGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarGetNextChild(hierarchy, childHierar)) {
+		PROC *child = HierarGetProc(childHierar);
 		if ((i_pgid == 0) || (child->pgid == i_pgid)) {
 			procSendSignal(child, i_signal);
 		}
@@ -518,8 +517,8 @@ STATIC PROC *searchProcPid(
 	HierarchyObj *childHierar;
 	PROC *child = NULL;
 
-	for (childHierar = HierarchyGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarchyGetNextChild(hierarchy, childHierar)) {
-		child = HierarchyGetProc(childHierar);
+	for (childHierar = HierarGetFirstChild(hierarchy); childHierar != NULL; childHierar = HierarGetNextChild(hierarchy, childHierar)) {
+		child = HierarGetProc(childHierar);
 		if (child->pid == i_pid) {
 			break;
 		}
@@ -534,6 +533,26 @@ STATIC PROC *searchProcPid(
 	return child;
 }
 
+/*
+ * ץinitץλҥץˤ
+ */
+STATIC void setParentToInit()
+{
+	PROC *this = getCurrentProc();
+	PROC *parent;
+	int eflag;
+
+	eflag = enterCli();
+	enter_spinlock(&spinLock);
+	{
+		parent = HierarGetParent(&this->hierarchy);
+		HierarRemoveChild(&parent->hierarchy, &this->hierarchy);
+		HierarAddChild(&initProc->hierarchy, &this->hierarchy);
+	}
+	exit_spinlock(&spinLock);
+	exitCli(eflag);
+}
+
 //================================== PUBLIC =============================================
 
 //--------------------------------------------------------------------------------------------------
@@ -700,58 +719,62 @@ ITIMER *getItimerProc()
 /*
  * ץλ
  *Գ
+ *ԳԲġ
  */
 void exit(
 	int state,		// ƥץϤ
 	int signum)
 {
-	static int exitSpinLock = 0;
+	PROC *this = getCurrentProc();
+	int procState;
 	int eflag;
 
+	// ¾ǥץơȤPROC_EXIT
 	eflag = enterCli();
-	enter_spinlock(&exitSpinLock);
+	enter_spinlock(&this->spinLock);
 	{
-		PROC *this = getCurrentProc();
-
-		if (this->state != PROC_EXIT) {
-			// åɤ˽λʥ
-			sendTaskMessage(this, (void*) TaskGetTaskSignal, (void*) sendSignal, SIGKILL, 0, 0, 0, 0, 0);
-
-			// üץγ
-			releaseTerm(this);
-
-			// եǥץγ
-			releaseFileStruct(getFileStructProc(this));
-
-			this->exit_state = state & 0xff;
-			this->signum = signum & 0xff;
-			
-			enter_spinlock(&spinLock);
-			{
-				PROC *parent;
-
-				// ҥץγ
-				releaseChild(this);
-
-				// ƤҥץINITץλҤɲä
-				addChildProc1(this);
-
-				parent = HierarchyGetParent(&this->hierarchy);
-				--parent->childCount;
-
-				// CPUƥץ­
-				getProcTime(this, &parent->ctime[0], &parent->ctime[1]);
-
-				procSendSignal(parent, SIGCHLD);
-				activeTaskSignal(getWaitTask(getMainThread(parent)));
-			}
-			exit_spinlock(&spinLock);
-
-			this->state = PROC_EXIT;
-		}
+		procState = this->state;
+		this->state = PROC_EXIT;
 	}
-	exit_spinlock(&exitSpinLock);
+	exit_spinlock(&this->spinLock);
 	exitCli(eflag);
+
+	// ץ
+	if (procState != PROC_EXIT) {
+		PROC *parent;
+
+		// åɤ˽λʥ
+		sendTaskMessage(this, (void*) TaskGetTaskSignal, (void*) sendSignal, SIGKILL, 0, 0, 0, 0, 0);
+
+		// üץγ
+		releaseTerm(this);
+
+		// եǥץγ
+		releaseFileStruct(getFileStructProc(this));
+
+		this->exit_state = state & 0xff;
+		this->signum = signum & 0xff;
+		
+//		eflag = enterCli();
+//		enter_spinlock(&spinLock);
+//		{
+			// ҥץγ
+			releaseChild(this);
+//		}
+//		exit_spinlock(&spinLock);
+//		exitCli(eflag);
+
+		// ƤҥץINITץλҤɲä
+		addChildProc1(this);
+
+		parent = HierarGetParent(&this->hierarchy);
+
+		// CPUƥץ­
+		getProcTime(this, &parent->ctime[0], &parent->ctime[1]);
+
+		procSendSignal(parent, SIGCHLD);
+		TaskAwake(getTaskWait(getMainThread(parent)));
+	}
 }
 
 /*
@@ -807,33 +830,21 @@ void setInitProc()
 	initProc = getCurrentProc();
 }
 
-/*
- * initProcwait
- */
-void waitProc1()
-{
-	ProcSignalSetMask(SIGCHLD);
-
-	for(;;) {
-		int eflag;
-		waitTaskSignal();
-
-		eflag = enterCli();
-		enter_spinlock(&spinLock);
-		{
-			releaseChild(initProc);
-		}
-		exit_spinlock(&spinLock);
-		exitCli(eflag);
-	}
-}
-
 /*
  * ͥ륹åɤfork
+ *ԳԲġ
+ * return : if parent process then child ID or error number, 
+ *          if child process then 0.
  */
 int forkKernelThread()
 {
-	return fork(1);
+	int error = fork(1);
+
+	if (error == 0) {
+		setParentToInit();
+	}
+
+	return error;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -850,12 +861,13 @@ int registThread(
 {
 	PROC *proc = (PROC*) m_proc;
 	AggregateList *aggregate = &proc->aggrThread;
+	AggregateListMethod *aggrMethod = &proc->aggrMethod;
 	int error = NOERR;
 
 	enter_spinlock(&proc->spinLock);
 	{
-		if (aggregate->getCount(aggregate) < PTHREAD_THREADS_MAX) {
-			aggregate->insertHead(aggregate, m_threadList);
+		if (aggrMethod->getCount(aggregate) < PTHREAD_THREADS_MAX) {
+			aggrMethod->insertHead(aggregate, m_threadList);
 		}
 		else {
 			error = -EAGAIN;
@@ -896,7 +908,8 @@ ThreadObj *getMainThread(
 	const void *i_proc)
 {
 	AggregateList *aggregate = &((PROC*) i_proc)->aggrThread;
-	return aggregate->refEnd(aggregate);
+	AggregateListMethod *aggrMethod = &((PROC*) i_proc)->aggrMethod;
+	return aggrMethod->refEnd(aggregate);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -945,7 +958,10 @@ int procSendSignal(
 	void *i_this,
 	const int i_signal)
 {
-	return sendTaskMessage(i_this, (void*) TaskGetTaskSignal, (void*) sendSignal, i_signal, 0, 0, 0, 0, 0);
+//**************************************************************************************************************
+//	return sendTaskMessage(i_this, (void*) TaskGetTaskSignal, (void*) sendSignal, i_signal, 0, 0, 0, 0, 0);
+	return sendTaskMessage(i_this, (void*) TaskGetTaskSignal, (void*) sendSignal, i_signal, 0, 0, 0, 0, *((uint*) &i_this - 1));
+//**************************************************************************************************************
 }
 
 /*
@@ -1010,26 +1026,6 @@ int procCreatUserHandlerStack(
 	return sendTaskMessage(i_this, (void*) getVmTask, (void*) vmCreatSignalHandlerStack, 0, 0, 0, 0, 0, 0);
 }
 
-/*
- * ץinitץλҥץˤ
- */
-void setParentToInit()
-{
-	PROC *this = getCurrentProc();
-	PROC *parent;
-	int eflag;
-
-	eflag = enterCli();
-	enter_spinlock(&spinLock);
-	{
-		parent = HierarchyGetParent(&this->hierarchy);
-		HierarchyRemoveChild(&parent->hierarchy, &this->hierarchy);
-		HierarchyAddChild(&initProc->hierarchy, &this->hierarchy);
-	}
-	exit_spinlock(&spinLock);
-	exitCli(eflag);
-}
-
 /*
  * Хå饦ɤǼ¹ԤƤ뤫
  *ԸƤդ⤷̤λҥץԤä顩
@@ -1038,13 +1034,24 @@ void setParentToInit()
 /*int isBackgroundProc()
 {
 	PROC *this = getCurrentProc();
-	PROC *parent = HierarchyGetParent(&this->hierarchy);
+	PROC *parent = HierarGetParent(&this->hierarchy);
 
 	ASSERT(parent != NULL);
 
 	return  (getTaskState(getMainThread(parent)) & TASK_SIGNAL_WAIT) ? NO : YES;
 }*/
 
+/*
+ * ƥץ򵯾
+ */
+void procWakeParent(
+	void *i_this)
+{
+	PROC *this = i_this;
+	PROC *parent = HierarGetParent(&this->hierarchy);
+	TaskAwake(getTaskWait(getMainThread(parent)));
+}
+
 //--------------------------------------------------------------------------------------------------
 // ƥॳ
 //--------------------------------------------------------------------------------------------------
@@ -1053,6 +1060,8 @@ void setParentToInit()
  * return : ƥץʤҥץID,ҥץʤ0,error=error number
  * todoꥹ
  *  ǥ쥯ȥꥹȥ꡼򥳥ԡ롣
+ * return : if parent process then child ID or error number, 
+ *          if child process then 0.
  */
 int sys_fork()
 {
@@ -1093,11 +1102,12 @@ int sys_wait(int pid, int *stat_loc, int options)
 		else {
 			rest = searchExitState(this, 0, -pid, state, stat_loc);
 		}
-		releaseChild(this);
 	}
 	exit_spinlock(&spinLock);
 	exitCli(eflag);
 
+	releaseChild(this);
+
 	if (0 < rest) {
 		return rest;
 	}
@@ -1106,7 +1116,7 @@ int sys_wait(int pid, int *stat_loc, int options)
 		return rest;
 	}
 
-	waitTaskSignal();
+	TaskWait();
 
 	eflag = enterCli();
 	enter_spinlock(&spinLock);
@@ -1123,11 +1133,12 @@ int sys_wait(int pid, int *stat_loc, int options)
 		else {
 			rest = searchExitState(this, 0, -pid, state, stat_loc);
 		}
-		releaseChild(this);
 	}
 	exit_spinlock(&spinLock);
 	exitCli(eflag);
 
+	releaseChild(this);
+
 	return rest;
 }
 
@@ -1176,7 +1187,7 @@ int sys_getpgid(
 int sys_getppid()
 {
 	PROC *this = getCurrentProc();
-	PROC *parent = HierarchyGetParent(&this->hierarchy);
+	PROC *parent = HierarGetParent(&this->hierarchy);
 	return parent->pid;
 }
 
@@ -1248,7 +1259,7 @@ int sys_setpgid(pid_t pid, pid_t pgid)
 				exitCli(eflag);
 				return -ESRCH;
 			}
-			parent = HierarchyGetParent(&proc->hierarchy);
+			parent = HierarGetParent(&proc->hierarchy);
 		}
 		exit_spinlock(&spinLock);
 		exitCli(eflag);
@@ -1283,6 +1294,7 @@ int sys_setpgid(pid_t pid, pid_t pgid)
 	}
 	exit_spinlock(&spinLock);
 	exitCli(eflag);
+
 	if (pgproc == NULL) {
 		return -EPERM;
 	}
@@ -1326,7 +1338,7 @@ int sys_times(struct tms *tms)
 {
 	PROC *proc;
 
-	if (checkMem(tms, sizeof(struct tms)) == ERR) {
+	if (vmIsReadArea(getVmProc(getCurrentProc()), tms, sizeof(struct tms)) == ERR) {
 		return -EFAULT;
 	}
 
@@ -1337,3 +1349,35 @@ int sys_times(struct tms *tms)
 
 	return 0;
 }
+
+int sysIsSetCtlterm()
+{
+	PROC *this = (PROC*) getCurrentProc();
+	return (this->ctlterm != NULL) ? YES : NO;
+}
+
+int sysSetCtlterm(
+	void *ctlterm)		// ⥸塼빽¤
+{
+	PROC *this = (PROC*) getCurrentProc();
+	this->ctlterm = ctlterm;
+	return 0;
+}
+
+//**************************************************************************************************
+#ifdef DEBUG
+static PROC *proc = NULL;
+
+void test_proc()
+{
+	if (proc == NULL) {
+		proc =getCurrentProc();
+	}
+	else {
+		if (proc->spinLock != 0) {
+			printk("                    test_proc %x-%x\n", proc, proc->spinLock);
+			idle();
+		}
+	}
+}
+#endif
