@@ -5,22 +5,14 @@
  * Distributed under the terms of the BSD License.
  */
 
-
-#ifndef system_h
-#define system_h
-
-
-#include <sys/unistd.h>
-
+#ifndef kern_system_h
+#define kern_system_h
 
 #ifndef ASM_FILE
 
-
 extern int sys_reboot(int);
 extern int sys_uname();
 
+#endif //ASM_FILE
 
-#endif
-
-
-#endif
+#endif //kern_system_h
