@@ -20,7 +20,7 @@
 
 extern void initTime();
 extern uint getNextIntervalTime(void);
-extern uint getCpuClock(void);
+extern uint calcCpuClock(void);
 extern void calcUserTime();
 extern void calcSysTime();
 extern void setTime(uint);
@@ -31,7 +31,6 @@ extern uint calcMiliSecondFromClock(uint64);
 extern uint64 calcMicroSecondFromClock(uint64);
 extern uint64 calcClockFromMicroSecond(uint64);
 extern uint64 calcClockFromMiliSecond(uint64);
-extern uint64 rdtsc();
 extern int sys_time();
 extern int sys_settime(uint);
 extern int sys_gettimeofday();
