@@ -14,12 +14,14 @@
 #include <sys/callout.h>
 #include <sys/systm.h>
 #include <sys/limits.h>
-#include <sys/AggregateList.h>
+#include <lib/IteratorList.h>
+#include <lib/AggregateList.h>
 #include <machine/lib.h>
+#include <machine/clock.h>
 #include <machine/interrupt.h>
+#include <machine/lock.h>
 #include <dev/8254.h>
 #include <dev/MC146818.h>
-#include <kern/lock.h>
 #include <kern/proc.h>
 #include <kern/ProcSignal.h>
 #include <kern/vm.h>
@@ -39,21 +41,24 @@
 	#define INLINE	inline
 #endif
 
-
 //==================================  ===========================================
 
 enum{
 	TIMER_MAX = 10,			// Υޡͭ
+	
+	// ޡե饰
+	FLG_NODELETE	= 1,		// ॢȸʤ
 };
 
 /*
  * ޡإå
  */
 typedef struct {
-	AggregateList	aggregate;					// ޡ󥯽
-	uint			(*getPastTime)();			// ޡߤޤǤθߤηвms
-	void			(*setTimer)(const uint);	// ޡms
-	int				lock;						// ԥå
+	AggregateList		aggrList;					// ޡ󥯽
+	AggregateListMethod	aggrMethod;					// 󥯥᥽å
+	uint				(*getPastTime)();			// ޡߤޤǤθߤηвms
+	void				(*setTimer)(const uint);	// ޡms
+	int					lock;						// ԥå
 } TIMER_HEAD;
 
 /*
@@ -64,158 +69,24 @@ typedef struct {
 	void		(*handle)(void*);	// ϥɥ
 	void		*arg;				// ϥɥ
 	List		ownedList;			// ͭ󥯥ꥹ
+	int			flag;				// ե饰
 
 	// ޡ
 	uint		count;				// Ĥ
 	List		timerList;			// ޡꥹ
 	TIMER_HEAD	*timerHead;			// ³Ƥ륿ޡإå³ƤʤNULL
-} TIMER;
+} Timer;
 
 //================================== PRIVATE =============================================
 
 /*
- * ޡΥ饤ե
- * allocTimer  registTimer  doTimer  doTimerˤ륿ॢ  λޤǥޡϳʤ
- *                                              or
- *                                         unregistTimer
+ * 顼ϥɥ
+ *Գ
  */
-
-/*
- * 󥿡Х륿ޡإå
- */
-static TIMER_HEAD intervalTimer;
-
-/*
- * ꥢ륿ޡإå
- */
-static TIMER_HEAD realTimer;
-
-/*
- * ƥॿޡͭإå
- */
-static TIMER_OWNER systemTimer;
-
-/*
- * ޡإåν
- */
-STATIC void initTimerHead()
+STATIC void handleAlarm(
+	void *proc)
 {
-	// 󥿡Х륿ޡإåν
-	AggregateListConstructor(&intervalTimer.aggregate);
-	intervalTimer.getPastTime = getIntervalTimerPastTime;
-	intervalTimer.setTimer = setIntervalTimer;
-	intervalTimer.lock = 0;
-	
-	// ꥢ륿ޡإåν
-	AggregateListConstructor(&realTimer.aggregate);
-	realTimer.getPastTime = getRealTimerPastTime;
-	realTimer.setTimer = setRealTimer;
-	realTimer.lock = 0;
-}
-
-/*
- * ޡ¤Τ򥿥ޡإå˻ֽ³롣
- *Թ
- */
-STATIC void linkTimer(
-	const int i_time,			// ޡms
-	TIMER *m_timer,				// ³륿ޡ¤
-	TIMER_HEAD *m_timerHead)	// ³西ޡإå
-{
-	uint time = i_time;
-	AggregateList *aggregate = &m_timerHead->aggregate;
-	IteratorList iterator;
-
-	aggregate->iterator(aggregate, &iterator);
-	for (;;) {
-		TIMER *timer;
-		
-		if (iterator.hasNext(&iterator) == BOOL_FALSE) {
-			aggregate->insertEnd(aggregate, &m_timer->timerList);
-			break;
-		}
-
-		timer = iterator.next(&iterator);
-		if (time <= timer->count) {
-			timer->count -= time;
-			aggregate->insertPrev(aggregate, &timer->timerList, &m_timer->timerList);
-			break;
-		}
-		else {
-			time -= timer->count;
-		}
-	}
-	m_timer->count = time;
-}
-
-/*
- * ޡ¤Τ򥿥ޡإåڤΥ
- *Թ
- */
-STATIC void unlinkTimer(
-	TIMER *m_timer,				// ڤΥޡ¤
-	TIMER_HEAD *m_timerHead)	// ³西ޡإå
-{
-	AggregateList *aggregate = &m_timerHead->aggregate;
-	TIMER *next = aggregate->refNext(aggregate, &m_timer->timerList);
-
-	ASSERT(m_timer != NULL);
-
-	if (next != NULL) {
-		next->count += m_timer->count;
-	}
-	aggregate->removeEntry(aggregate, &m_timer->timerList);
-}
-
-/*
- * ֤вᤷޡ򥿥ޡإåڤΥƼ롣
- *ԳաԹ
- */
-STATIC void getTimeoutTimer(
-	const uint i_time,						// вms
-	TIMER_HEAD *m_timerHead,				// ³西ޡإå
-	AggregateList *m_aggregateUnlinkList)	// ޡ¤Τ³ñꥹȥإå
-{
-	uint time = i_time;
-	AggregateList *aggregate = &m_timerHead->aggregate;
-	TIMER *timer;
-
-	for (timer = aggregate->refHead(aggregate); timer != NULL;) {
-		TIMER *next = aggregate->refNext(aggregate, &timer->timerList);
-
-		if (timer->count <= time) {
-			aggregate->removeEntry(aggregate, &timer->timerList);
-			time -= timer->count;
-			timer->count = 0;
-			m_aggregateUnlinkList->insertHead(m_aggregateUnlinkList, &timer->timerList);
-		}
-		else {
-			timer->count -= time;
-			break;
-		}
-		
-		timer = next;
-	}
-}
-
-/*
- * ޡ󥯤֤򸺤餹
- *Թ
- */
-STATIC void subtractTimeTimerLink(
-	const uint time,			//  ms
-	TIMER_HEAD *i_timerHead)	// ³西ޡإå
-{
-	AggregateList *aggregate = &i_timerHead->aggregate;
-	TIMER *timer = aggregate->refHead(aggregate);
-	
-	if (timer == NULL) {
-		return;
-	}
-
-	ASSERT(time <= timer->count);
-	
-	timer->count -= time;
+	procSendSignal(proc, SIGALRM);
 }
 
 /*
@@ -224,8 +95,9 @@ STATIC void subtractTimeTimerLink(
 STATIC void setTimeTimerLink(
 	TIMER_HEAD *i_timerHead)	// ³西ޡإå
 {
-	AggregateList *aggregate = &i_timerHead->aggregate;
-	TIMER *timer = aggregate->refHead(aggregate);
+	AggregateList *aggrList = &i_timerHead->aggrList;
+	AggregateListMethod *aggrMethod = &i_timerHead->aggrMethod;
+	Timer *timer = aggrMethod->refHead(aggrList);
 	
 	if (timer == NULL) {
 		return;
@@ -240,98 +112,177 @@ STATIC void setTimeTimerLink(
 	}
 }
 
+//--------------------------------------------------------------------------------------------------
+// ޡ
+//--------------------------------------------------------------------------------------------------
+
 /*
- * ޡϿե饰ꤹ
+ * 󥿡Х륿ޡإå
  */
-STATIC INLINE void setRegist(
-	TIMER *m_timer,
-	TIMER_HEAD *timerHead)
+static TIMER_HEAD intervalTimer;
+
+/*
+ * ꥢ륿ޡإå
+ */
+static TIMER_HEAD realTimer;
+
+/*
+ * ޡإåν
+ */
+STATIC void initTimerHead()
 {
-	m_timer->timerHead = timerHead;
+	// 󥿡Х륿ޡإåν
+	AggregateListConstructor(&intervalTimer.aggrList, &intervalTimer.aggrMethod);
+	intervalTimer.getPastTime = getIntervalTimerPastTime;
+	intervalTimer.setTimer = setIntervalTimer;
+	intervalTimer.lock = 0;
+	
+	// ꥢ륿ޡإåν
+	AggregateListConstructor(&realTimer.aggrList, &realTimer.aggrMethod);
+	realTimer.getPastTime = getRealTimerPastTime;
+	realTimer.setTimer = setRealTimer;
+	realTimer.lock = 0;
 }
 
 /*
- * ޡϿե饰
+ * ޡ¤Τ򥿥ޡإå˻ֽ³롣
+ *ճ߶ػߡޡإåȤ˥å򤫤ƸƤӽФ뤳
  */
-STATIC INLINE void resetRegist(
-	TIMER *m_timer)
+STATIC void linkTimer(
+	Timer *this,				// ³륿ޡ¤
+	TIMER_HEAD *m_timerHead,	// ³西ޡإå
+	const int i_time)			// ޡms
 {
-	m_timer->timerHead = NULL;
-}
-
-/*
- * ޡ󥯤ϿƤ뤫
- *Ի
- * return : YES or NO
- */
-STATIC INLINE int isRegistTimer(
-	const TIMER *timer)
-{
-	return (timer->timerHead != NULL) ? YES : NO;
-}
-
-/*
- *ԳաԹ
- * ֤вᤷޡϥɥ¹Ԥ롣
- */
-STATIC void doTimer(
-	const uint i_time,			// в
-	TIMER_HEAD *m_timerHead)	// ³إå
-{
-	AggregateList aggregateUnlilnkTimer;	// ñꥹ
+	uint time = i_time;
+	AggregateList *aggrList = &m_timerHead->aggrList;
+	AggregateListMethod *aggrMethod = &m_timerHead->aggrMethod;
 	IteratorList iterator;
-	int eflags;
 
-	AggregateSingleListConstructor(&aggregateUnlilnkTimer);
-
-	eflags = enterCli();
-	enter_spinlock(&m_timerHead->lock);
-	{
-		// ॢȥޡμ
-		getTimeoutTimer(i_time, m_timerHead, &aggregateUnlilnkTimer);
-	}
-	exit_spinlock(&m_timerHead->lock);
-	exitCli(eflags);
-
-	// ޡϥɥμ¹
-	aggregateUnlilnkTimer.iterator(&aggregateUnlilnkTimer, &iterator);
-	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-		TIMER *timer = iterator.next(&iterator);
-
-		timer->handle(timer->arg);
-	}
-
-	eflags = enterCli();
-	enter_spinlock(&m_timerHead->lock);
-	{
-		TIMER *timer;
-
-		while ((timer = aggregateUnlilnkTimer.getHead(&aggregateUnlilnkTimer)) != NULL) {
-			// ³ޡͤк³
-			if (0 < timer->init) {
-				linkTimer(timer->init, timer, m_timerHead);
-				setRegist(timer, m_timerHead);
-			}
-			else {
-				resetRegist(timer);
-			}
+	IteratorListConstruct(&iterator, aggrList, aggrMethod);
+	for (;;) {
+		Timer *timer;
+		
+		if (iterator.hasNext(&iterator) == BOOL_FALSE) {
+			aggrMethod->insertEnd(aggrList, &this->timerList);
+			break;
 		}
 
-		// ǥХޡ֤
-		setTimeTimerLink(m_timerHead);
+		timer = iterator.next(&iterator);
+		if (time <= timer->count) {
+			timer->count -= time;
+			aggrMethod->insertPrev(aggrList, &timer->timerList, &this->timerList);
+			break;
+		}
+		else {
+			time -= timer->count;
+		}
 	}
-	exit_spinlock(&m_timerHead->lock);
-	exitCli(eflags);
+	this->count = time;
+}
+
+/*
+ * ޡ¤Τ򥿥ޡإåڤΥ
+ *ճ߶ػߡޡإåȤ˥å򤫤ƸƤӽФ뤳
+ */
+STATIC void unlinkTimer(
+	Timer *this,				// ڤΥޡ¤
+	TIMER_HEAD *i_timerHead)	// ³西ޡإå
+{
+	AggregateList *aggrList = &i_timerHead->aggrList;
+	AggregateListMethod *aggrMethod = &i_timerHead->aggrMethod;
+
+	if (i_timerHead->aggrMethod.isLink(&i_timerHead->aggrList, &this->timerList) == YES) {
+		Timer *next = aggrMethod->refNext(aggrList, &this->timerList);
+		if (next != NULL) {
+			next->count += this->count;
+		}
+		aggrMethod->removeEntry(aggrList, &this->timerList);
+	}
+}
+
+/*
+ * ֤вᤷޡ򥿥ޡإåڤΥƼ롣
+ *ճ߶ػߡޡإåȤ˥å򤫤ƸƤӽФ뤳
+ */
+STATIC void getTimeoutTimer(
+	const uint i_time,						// вms
+	TIMER_HEAD *m_timerHead,				// ³西ޡإå
+	AggregateList *aggrUnlinkList,			// ޡ¤Τ³ñꥹȥإå
+	AggregateListMethod *aggrUnlinkMehotd)	// ޡ¤Τ³ñꥹȥ᥽å
+{
+	uint time = i_time;
+	AggregateList *aggrList = &m_timerHead->aggrList;
+	AggregateListMethod *aggrMethod = &m_timerHead->aggrMethod;
+	Timer *timer;
+
+	for (timer = aggrMethod->refHead(aggrList); timer != NULL;) {
+		Timer *next = aggrMethod->refNext(aggrList, &timer->timerList);
+
+		if (timer->count <= time) {
+			aggrMethod->removeEntry(aggrList, &timer->timerList);
+			time -= timer->count;
+			timer->count = 0;
+			aggrUnlinkMehotd->insertHead(aggrUnlinkList, &timer->timerList);
+		}
+		else {
+			timer->count -= time;
+			break;
+		}
+		
+		timer = next;
+	}
+}
+
+/*
+ * ޡ󥯤֤򸺤餹
+ *ճ߶ػߡޡإåȤ˥å򤫤ƸƤӽФ뤳
+ */
+STATIC void subtractTimeTimerLink(
+	TIMER_HEAD *i_timerHead,	// ³西ޡإå
+	const uint time)			//  ms
+{
+	AggregateList *aggrList = &i_timerHead->aggrList;
+	AggregateListMethod *aggrMethod = &i_timerHead->aggrMethod;
+	Timer *timer = aggrMethod->refHead(aggrList);
+	
+	if (timer == NULL) {
+		return;
+	}
+
+	ASSERT(time <= timer->count);
+	
+	timer->count -= time;
+}
+
+//-------------------------------------- Ͽ ----------------------------------------------
+
+/*
+ * ޡ󥯤³Ƥ뤫
+ *ճ߶ػߤǸƤӽФ뤳
+ * return : YES or NO
+ */
+STATIC int isRegistTimer(
+	const Timer *this)
+{
+	TIMER_HEAD *timerHead = this->timerHead;
+
+	ASSERT(this->timerHead != NULL);
+	
+	if (timerHead->aggrMethod.isLink(&timerHead->aggrList, &this->timerList) == YES) {
+		return YES;
+	}
+	return (this->flag != 0) ? YES : NO;
 }
 
 /*
  * ޡ¤Τ򥿥ޡإåϿ
- *Թ
+ *ճ߶ػߤǸƤӽФ뤳
  * return : error number
  */
 STATIC void registTimer(
+	Timer  *this,
 	const uint i_time,	// ޡms
-	TIMER  *m_timer)
+	const int flag)
 {
 	TIMER_HEAD *timerHead;
 	int eflags;
@@ -353,67 +304,127 @@ STATIC void registTimer(
 		pastTime = timerHead->getPastTime();
 
 		// ޡ󥯤³
-		subtractTimeTimerLink(pastTime, timerHead);
-		linkTimer(i_time, m_timer, timerHead);
-		setRegist(m_timer, timerHead);
+		subtractTimeTimerLink(timerHead, pastTime);
+		linkTimer(this, timerHead, i_time);
+		this->timerHead = timerHead;
 
 		// ǥХޡ֤
 		setTimeTimerLink(timerHead);
 	}
 	exit_spinlock(&timerHead->lock);
 	exitCli(eflags);
+	
+	this->flag = flag;
 }
 
 /*
  * ޡ¤Τ򥿥ޡإå
  */
 STATIC void unregistTimer(
-	TIMER  *m_timer)
+	Timer  *this)
 {
-	ASSERT(m_timer != NULL);
-	
-	if (isRegistTimer(m_timer) == YES) {
-		TIMER_HEAD *timerHead = m_timer->timerHead;
-		int eflags;
+	TIMER_HEAD *timerHead = this->timerHead;
+	int eflags;
 
-		eflags = enterCli();
-		enter_spinlock(&timerHead->lock);
-		{
-			unlinkTimer(m_timer, m_timer->timerHead);
-			resetRegist(m_timer);
-		}
-		exit_spinlock(&timerHead->lock);
-		exitCli(eflags);
+	eflags = enterCli();
+	enter_spinlock(&timerHead->lock);
+	{
+		unlinkTimer(this, timerHead);
+		this->flag = 0;
 	}
+	exit_spinlock(&timerHead->lock);
+	exitCli(eflags);
 }
 
+//-------------------------------------- ޡ ----------------------------------------------
+
+/*
+ * ֤вᤷޡϥɥ¹Ԥ롣
+ *ԳաԹ
+ */
+STATIC void doTimer(
+	const uint i_time,			// в
+	TIMER_HEAD *m_timerHead)	// ³إå
+{
+	AggregateList aggrListUnlilnkTimer;				// ñꥹ
+	AggregateListMethod aggrMethodUnlilnkTimer;
+	IteratorList iterator;
+	int eflags;
+
+	AggregateListConstructor(&aggrListUnlilnkTimer, &aggrMethodUnlilnkTimer);
+
+	eflags = enterCli();
+	enter_spinlock(&m_timerHead->lock);
+	{
+		// ॢȥޡμ
+		getTimeoutTimer(i_time, m_timerHead, &aggrListUnlilnkTimer, &aggrMethodUnlilnkTimer);
+	}
+	exit_spinlock(&m_timerHead->lock);
+	exitCli(eflags);
+
+	// ޡϥɥμ¹
+	IteratorListConstruct(&iterator, &aggrListUnlilnkTimer, &aggrMethodUnlilnkTimer);
+	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
+		Timer *timer = iterator.next(&iterator);
+
+		timer->handle(timer->arg);
+	}
+
+	eflags = enterCli();
+	enter_spinlock(&m_timerHead->lock);
+	{
+		Timer *timer;
+
+		while ((timer = aggrMethodUnlilnkTimer.getHead(&aggrListUnlilnkTimer)) != NULL) {
+			// ³ޡͤк³
+			if (0 < timer->init) {
+				linkTimer(timer, m_timerHead, timer->init);
+				timer->timerHead = m_timerHead;
+			}
+		}
+
+		// ǥХޡ֤
+		setTimeTimerLink(m_timerHead);
+	}
+	exit_spinlock(&m_timerHead->lock);
+	exitCli(eflags);
+}
+
+//--------------------------------------------------------------------------------------------------
+// ޡͭ
+//--------------------------------------------------------------------------------------------------
+
+/*
+ * ƥॿޡͭإå
+ */
+static TIMER_OWNER systemTimer;
+
 /*
  * ޡͭإåν
  */
 STATIC INLINE void initOwnerHead(
 	TIMER_OWNER *m_owner)
 {
-	AggregateSingleListConstructor(&m_owner->aggregate);
-	m_owner->count = 0;
+	AggregateListConstructor(&m_owner->aggrList, &m_owner->aggrMethod);
 	m_owner->lock = 0;
 }
 
 /*
  * ͭإåѥޡõ
+ *ճ߶ػߤǸƤФ뤳
  * return : NOERR or -ENOENT
  */
 STATIC int findDelTimer(
 	TIMER_OWNER *i_owner,		// ͭإå
-	TIMER **o_timer)
+	Timer **o_timer)
 {
-	AggregateList *aggregate = &i_owner->aggregate;
 	IteratorList iterator;
 
-	aggregate->iterator(aggregate, &iterator);
+	IteratorListConstruct(&iterator, &i_owner->aggrList, &i_owner->aggrMethod);
 	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-		TIMER *timer = iterator.next(&iterator);
+		Timer *timer = iterator.next(&iterator);
 		
-		if (isRegistTimer(timer) == NO) {
+		if ((isRegistTimer(timer) == NO) && ((timer->flag & FLG_NODELETE) == 0)) {
 			*o_timer = timer;
 			return NOERR;
 		}
@@ -424,6 +435,7 @@ STATIC int findDelTimer(
 
 /*
  * ͭ󥯤˥ޡƤƤʤп˥ޡơޡ롣
+ *ճ߶ػߤǸƤФ뤳
  * return : error number
  */
 STATIC int allocTimer(
@@ -431,17 +443,16 @@ STATIC int allocTimer(
 	void (*handle)(void*),	// ַв˼¹Ԥ
 	void *arg,				// ַв˼¹Ԥΰ
 	TIMER_OWNER *m_owner,	// ޡͭ
-	TIMER **o_timer)
+	Timer **o_timer)
 {
-	TIMER *timer;
-
-	ASSERT(handle != NULL);
+	Timer *timer;
 
 	// ޡõƤʤгƤ
 	if (findDelTimer(m_owner, &timer) == -ENOENT) {
-		AggregateList *aggregate = &m_owner->aggregate;
+		AggregateList *aggrList = &m_owner->aggrList;
+		AggregateListMethod *aggrMethod = &m_owner->aggrMethod;
 
-		if (TIMER_MAX <= m_owner->count) {
+		if (TIMER_MAX <= aggrMethod->getCount(aggrList)) {
 			WARNING(0);
 			return -ENOMEM;
 		}
@@ -451,8 +462,7 @@ STATIC int allocTimer(
 			return -ENOMEM;
 		}
 		listConstructor(&timer->ownedList, timer);
-		aggregate->insertHead(aggregate, &timer->ownedList);
-		++m_owner->count;
+		aggrMethod->insertHead(aggrList, &timer->ownedList);
 	}
 
 	listConstructor(&timer->timerList, timer);
@@ -466,16 +476,86 @@ STATIC int allocTimer(
 }
 
 /*
- * 顼ϥɥ
- *Գ
+ * ͭ󥯤
+ *ճ߶ػߤǸƤФ뤳
+ *unregistTimer()ϿƤ뤳
+ * return : error number
  */
-STATIC void handleAlarm(void *proc)
+/*
+STATIC void freeTimer(
+	Timer *this,
+	TIMER_OWNER *i_owner)
 {
-	procSendSignal(proc, SIGALRM);
+	AggregateList *aggrList = &i_owner->aggrList;
+	AggregateListMethod *aggrMethod = &i_owner->aggrMethod;
+
+	enter_spinlock(&systemTimer.lock);
+	{
+		aggrMethod->removeEntry(aggrList, &this->ownedList);
+	}
+	exit_spinlock(&systemTimer.lock);
+	kfree(this);
+}
+*/
+
+/*
+ * ͭإåϥɥõ
+ *ճ߶ػߤǸƤФ뤳
+ * return : Timer or NULL
+ */
+STATIC Timer *findTimerByHandle(
+	TIMER_OWNER *i_owner,		// ͭإå
+	void (*handle)(void*))
+{
+	IteratorList iterator;
+
+	IteratorListConstruct(&iterator, &i_owner->aggrList, &i_owner->aggrMethod);
+	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
+		Timer *timer = iterator.next(&iterator);
+		
+		if (timer->handle == handle) {
+			return timer;
+		}
+	}
+
+	return NULL;
 }
 
 //================================== PUBLIC ==============================================
 
+/*
+ * ޡν
+ *ե֡Ȼ˸ƤӽФ
+ */
+void initTimer(void)
+{
+#ifdef DEBUG
+	if (sizeof(Timer) != sizeof(TimerObj)) {
+		printk("initTimer() object size error! Timer=%d TimerObj=%d\n", sizeof(Timer), sizeof(TimerObj));
+		idle();
+	}
+#endif
+	// ƥॿޡإåν
+	initTimerHead();
+
+	// ƥॿޡͭإåν
+	initOwnerHead(&systemTimer);
+}
+
+/*
+ * ץޡν
+ *եץ˸ƤӽФ
+ */
+void initProcessTimer(
+	void *m_timer)
+{
+	initOwnerHead(m_timer);
+}
+
+//--------------------------------------------------------------------------------------------------
+// 
+//--------------------------------------------------------------------------------------------------
+
 /*
  *Գ
  * 󥿡Х륿ޡμ¹
@@ -496,6 +576,10 @@ void doRealTimer(
 	doTimer(time, &realTimer);
 }
 
+//--------------------------------------------------------------------------------------------------
+// ޡϿ
+//--------------------------------------------------------------------------------------------------
+
 /*
  * ƥॿޡƤ
  * return : error number
@@ -511,12 +595,12 @@ int setSystemTimer(
 
 	enter_spinlock(&systemTimer.lock);
 	{
-		error = allocTimer(initTime, handle, arg, &systemTimer, (TIMER**) o_timer);
+		error = allocTimer(initTime, handle, arg, &systemTimer, (Timer**) o_timer);
 	}
 	exit_spinlock(&systemTimer.lock);
 	
 	if (error == NOERR) {
-		registTimer(i_time, *o_timer);
+		registTimer(*o_timer, i_time, 0);
 		return NOERR;
 	}
 	else {
@@ -525,7 +609,6 @@ int setSystemTimer(
 }
 
 /*
- *Թ
  * ץޡƤ
  * return : error number
  */
@@ -536,24 +619,76 @@ int setTaskTimer(
 	void *arg,				// ַв˼¹Ԥΰ
 	void **o_timer)
 {
-	int error = allocTimer(initTime, handle, arg, getTaskTimer(getCurrentTask()), (TIMER**)o_timer);
-	if (error != NOERR) {
-		return error;
-	}
-	registTimer(i_time, *o_timer);
+	int eflags;
+	int error = NOERR;
 
-	return NOERR;
+	eflags = enterCli();
+	{
+		error = allocTimer(initTime, handle, arg, getTaskTimer(getCurrentTask()), (Timer**)o_timer);
+		if (error == NOERR) {
+			registTimer(*o_timer, i_time, FLG_NODELETE);
+		}
+	}
+	exitCli(eflags);
+
+	return error;
+}
+
+/*
+ * 顼ॿޡƤ
+ * return : error number
+ */
+int setAlarmTimer(
+	const uint i_time,		// ޡms
+	const uint initTime,	// ַв˺Ƥꤹ륿ޡms
+	TimerObj *m_Timer)		// Υ顼ॿޡ
+{
+	int error;
+	int eflags;
+
+	eflags = enterCli();
+	{
+		Timer *timer;
+
+		// 顼ॿޡˤв
+		timer = findTimerByHandle(getTaskTimer(getCurrentTask()), handleAlarm);
+		if (timer != NULL) {
+			*m_Timer = *(TimerObj*) timer;
+			unregistTimer(timer);
+		}
+		else {
+			Timer *nullTimer = (Timer*) m_Timer;
+			nullTimer->init = 0;
+			nullTimer->timerHead = NULL;
+		}
+
+		// ޡ
+		if (0 < i_time){
+			error = allocTimer(initTime, handleAlarm, getCurrentProc(), getTaskTimer(getCurrentTask()), (Timer**) &timer);
+			if (error == NOERR) {
+				registTimer(timer, i_time, 0);
+			}
+		}
+		else {
+			error = NOERR;
+		}
+	}
+	exitCli(eflags);
+
+	return error;
 }
 
 /*
  * ޡ
  * return : error number
  */
-void freeTimer(
+void resetTimer(
 	void *m_timer)
 {
+	Timer *timer = (Timer*) m_timer;
+
 	// Ͽ
-	unregistTimer(m_timer);
+	unregistTimer(timer);
 }
 
 /*
@@ -563,25 +698,27 @@ void freeTimer(
 void releaseProcessTimer(
 	TIMER_OWNER *m_owner)
 {
-	AggregateList *aggregate = &m_owner->aggregate;
-	TIMER *timer;
+	Timer *timer;
 
-	while ((timer = aggregate->getHead(aggregate)) != NULL) {
+	while ((timer = m_owner->aggrMethod.getHead(&m_owner->aggrList)) != NULL) {
 		// Ͽ
 		unregistTimer(timer);
-
 		kfree(timer);
 	}
 }
 
+//--------------------------------------------------------------------------------------------------
+// GETTER
+//--------------------------------------------------------------------------------------------------
+
 /*
  * ֤
  * return : ms
  */
 uint getInitTime(
-	void *i_timer)
+	TimerObj *i_timer)
 {
-	TIMER *timer = i_timer;
+	Timer *timer = (Timer*) i_timer;
 	return timer->init;
 }
 
@@ -591,69 +728,70 @@ uint getInitTime(
  * return : Ĥms
  */
 uint getRemainTime(
-	void *i_timer)
+	TimerObj *i_timer)
 {
-	TIMER *timer = i_timer;
-	TIMER_HEAD *timerHead = timer->timerHead;
+	Timer *this = (Timer*) i_timer;
+	TIMER_HEAD *timerHead = this->timerHead;
 	uint time;
 	int eflags;
-	
-	ASSERT(timer != NULL);
+
+	if (timerHead == NULL) {
+		return 0;
+	}
 
 	time = 0;
-	if (isRegistTimer(timer) == YES) {
-		eflags = enterCli();
-		enter_spinlock(&timerHead->lock);
-		{
-			if (isRegistTimer(timer) == YES) {
-				AggregateList *aggregate = &timerHead->aggregate;
-				IteratorList iterator;
+	eflags = enterCli();
+	enter_spinlock(&timerHead->lock);
+	{
+		if (timerHead->aggrMethod.isLink(&timerHead->aggrList, &this->timerList) == YES) {
+			IteratorList iterator;
 
-				aggregate->iterator(aggregate, &iterator);
-				while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-					TIMER *next = iterator.next(&iterator);
+			IteratorListConstruct(&iterator, &timerHead->aggrList, &timerHead->aggrMethod);
+			while (iterator.hasNext(&iterator) == BOOL_TRUE) {
+				Timer *next = iterator.next(&iterator);
 
-					// Ĥ֤û
-					time += next->count;
-					if (next == timer) {
-						break;
-					}
+				// Ĥ֤û
+				time += next->count;
+				if (next == this) {
+					break;
 				}
 			}
 		}
-		exit_spinlock(&timerHead->lock);
-		exitCli(eflags);
 	}
+	exit_spinlock(&timerHead->lock);
+	exitCli(eflags);
 
 	return time;
 }
 
 /*
- * ͭإåϥɥõ
- *Ի
- * return : NOERR or -ENOENT
+ * 顼ॿޡ
  */
-int findTimerByHandle(
-	TIMER_OWNER *i_owner,		// ͭإå
-	void (*handle)(void*),
-	void **o_timer)
+void getAlarmTimer(
+	TimerObj *m_timer)
 {
-	AggregateList *aggregate = &i_owner->aggregate;
-	IteratorList iterator;
+	Timer *timer;
+	int eflags;
 
-	aggregate->iterator(aggregate, &iterator);
-	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-		TIMER *timer = iterator.next(&iterator);
-		
-		if (timer->handle == handle) {
-			*o_timer = timer;
-			return NOERR;
-		}
+	eflags = enterCli();
+	{
+		timer = findTimerByHandle(getTaskTimer(getCurrentTask()), handleAlarm);
+	}
+	exitCli(eflags);
+	if (timer != NULL) {
+		*m_timer = *(TimerObj*) timer;
+	}
+	else {
+		timer = (Timer*) m_timer;
+		timer->init = 0;
+		timer->timerHead = NULL;
 	}
-
-	return -ENOENT;
 }
 
+//--------------------------------------------------------------------------------------------------
+// ¾ޡ
+//--------------------------------------------------------------------------------------------------
+
 /*
  * timeout --
  *	Execute a function after a specified length of time.
@@ -710,11 +848,12 @@ void untimeout(
 	void *arg, 
 	struct callout_handle callout)
 {
-	if (callout.callout == NULL){
+	Timer *this = callout.callout;
+
+	if (this == NULL) {
 		return;
 	}
-
-	freeTimer(callout.callout);
+	resetTimer(callout.callout);
 }
 
 void callout_init(void *c)
@@ -736,12 +875,6 @@ int callout_stop(void *c)
 	return 0;
 }
 
-/***************************************************************************************************
- * ¾ޡ
- ***************************************************************************************************/
-
-//================================== PUBLIC =============================================
-
 /*
  * micro seccond ޡ
  */
@@ -750,9 +883,9 @@ void micro_timer(
 {
 	uint64 end;
 
-	end = rdtsc();
+	end = getCpuClock();
 	end += calcClockFromMicroSecond(count);
-	while (rdtsc() < end);
+	while (getCpuClock() < end);
 }
 
 /*
@@ -763,36 +896,9 @@ void mili_timer(
 {
 	uint64 end;
 
-	end = rdtsc();
+	end = getCpuClock();
 	end += calcClockFromMiliSecond(count);
-	while (rdtsc() < end);
-}
-
-/***************************************************************************************************
- * 
- ***************************************************************************************************/
-
-/*
- * ޡν
- *ե֡Ȼ˸ƤӽФ
- */
-void initTimer(void)
-{
-	// ƥॿޡإåν
-	initTimerHead();
-
-	// ƥॿޡͭإåν
-	initOwnerHead(&systemTimer);
-}
-
-/*
- * ץޡν
- *եץ˸ƤӽФ
- */
-void initProcessTimer(
-	void *m_timer)
-{
-	initOwnerHead(m_timer);
+	while (getCpuClock() < end);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -802,20 +908,10 @@ void initProcessTimer(
 int sys_alarm(
 	uint seconds)
 {
-	void *timer;
+	TimerObj dummy;
+	int error = setAlarmTimer(seconds * 1000, 0, &dummy);
 
-	// 顼ॿޡˤв
-	if (findTimerByHandle(getTaskTimer(getCurrentTask()), handleAlarm, &timer) == NOERR) {
-		freeTimer(timer);
-	}
-
-	// ޡ
-	if (0 < seconds){
-		return setTaskTimer(seconds * 1000, 0, handleAlarm, getCurrentProc(), &timer);
-	}
-	else {
-		return 0;
-	}
+	return error;
 }
 
 void sysMicroDelay(
@@ -826,16 +922,10 @@ void sysMicroDelay(
 
 //*************************************************************************************************
 
-void test_timer()
+#ifdef DEBUG
+void test_timer(
+	int num)
 {
-	AggregateList *aggregate = &intervalTimer.aggregate;
-	IteratorList iterator;
-
-	printk("test_timer ");
-	aggregate->iterator(aggregate, &iterator);
-	while (iterator.hasNext(&iterator) == BOOL_TRUE) {
-		TIMER *timer = iterator.next(&iterator);
-		printk("%d ", timer->init);
-	}
-	printk("\n");
+	printDebug(80 * 10 + 70, "test-%d  ", num);
 }
+#endif
