@@ -11,7 +11,7 @@
 #include <sys/config.h>
 #include <sys/types.h>
 #include <sys/param.h>
-#include <sys/List.h>
+#include <lib/List.h>
 
 #include <kern/debug.h>
 
