@@ -7,17 +7,16 @@
  * ͥ饤֥꡼
  */
 
-
 #include <sys/config.h>
 #include <sys/types.h>
+#include <sys/limits.h>
 #include <sys/tty.h>
 #include <sys/libkern.h>
 #include <machine/stdarg.h>
-#include <lib/lib.h>
 #include <kern/time.h>
 #include <kern/vm.h>
 #include <dev/console/console.h>
-
+#include <lib/lib.h>
 
 /****************************************************************************
  *
@@ -191,7 +190,7 @@ int iToBinary(uint value,char *m_str)
 #define TOCONS	0x01
 #define TOTTY	0x02
 #define TOLOG	0x04
-#define MAXNBUF	(sizeof(quad_t) * BYTE_BITNUM + 1)
+#define MAXNBUF	(sizeof(quad_t) * CHAR_BIT + 1)
 struct	tty *constty;			/* pointer to console "window" tty */
 
 struct putchar_arg {
@@ -667,9 +666,11 @@ int sprintf(char *buf, const char *cfmt, ...)
 }
 
 /*
- *FreeBSD
+ * ͥѥ˥å
  */
-void panic(const char *ctl, ...)
+void panic(
+	const char *ctl, 
+	...)
 {
 	va_list va;
 
@@ -679,7 +680,7 @@ void panic(const char *ctl, ...)
 	vprintf("\n", NULL);
 	va_end(va);
 
-	sys_exit(1);
+	idle();
 }
 
 /***********************************************************************************
@@ -862,6 +863,17 @@ LOOP:
 	}
 }
 
+/*
+ * ӥåȤåȤƤ뤫
+ * return : 0ʳ or 0
+ */
+int bitIsSet(
+	uint *dst,
+	const uint bitFlag)
+{
+	return *dst & bitFlag;
+}
+
 /***************************************************************************************
  *
  *ǥХå
@@ -878,7 +890,6 @@ void printDebug(int column,const char *str,...)
 	int size;
 	void *arg = &str + 1;
 
-
 	displayTop(column, "          ", 10);
 	size = formatString(str,buf,arg);
 	displayTop(column, buf, size);
