@@ -16,11 +16,11 @@
 #ifndef ASM_FILE
 
 
-extern int			cmpPath(const char *name,const char *path,int count);
+extern int			pathCmpName(const char *name, const char *path, const int count);
 extern size_t		pathLen(const char *);
 extern int			cmpStrNum(const char *,const char *);
 extern int			isLastPath(const char *);
-extern const char	*getNextPath(const char *);
+extern const char	*pathGetNext(const char *);
 extern void			convertPath(const char *, char *);
 
 
