@@ -5,25 +5,20 @@
  * Distributed under the terms of the BSD License.
  */
 
-
 #ifndef module_interrupt_h
 #define module_interrupt_h
 
-
-#include <sys/types.h>
-#include <kern/Thread.h>
-
+#include <sys/Thread.h>
 
 #ifndef ASM_FILE
 
-
 extern ThreadObj *IntrDscGetTask(const int);
-extern inthand2_t *IntrDscGetHandler(const int);
 extern void *IntrDscGetArg(const int);
 extern void *IntrDscGetReturnMethod(const int);
+extern void *IntrDscGetStackPoint(const int);
+extern int sysIntrEstablish();
+extern int sysIntrDisestablish();
 
+#endif // ASM_FILE
 
-#endif
-
-
-#endif
+#endif // module_interrupt_h
