@@ -13,12 +13,13 @@
 #include <sys/param.h>
 #include <dev/console/console.h>
 #include <lib/lib.h>
+#include <machine/clock.h>
 #include <machine/interrupt.h>
-#include <kern/errno.h>
+#include <machine/lock.h>
 #include <kern/kmalloc.h>
-#include <kern/Thread.h>
+#include <sys/Thread.h>
+#include <kern/TaskWait.h>
 #include <kern/time.h>
-#include <kern/lock.h>
 #include <kern/ProcSignal.h>
 #include <kern/fs.h>
 #include <kern/timer.h>
@@ -123,6 +124,10 @@ STATIC INLINE void setPseudoHead(
 
 //-----------------------------  -------------------------------------------
 
+//************************************************************************
+static uint seqnumber;
+//************************************************************************
+
 /*
  * TCPȤ
  *աեץǡɬintǳڤ륵ˤ뤳ȡ
@@ -266,7 +271,7 @@ static int cnt = 0;
 			// ǡ򥽥åȤϤ
 			putRecvBufTcp(mbuf, &sock->recvBuf, &sock->lockGate);
 			if (sock->waitTask != NULL) {
-				activeTaskSoonSignal(getWaitTask(sock->waitTask));
+				TaskAwakeSoon(getTaskWait(sock->waitTask));
 			}
 			return 1;
 		}
@@ -276,7 +281,7 @@ static int cnt = 0;
 	for (sock = waitLstnSc.next; sock != &waitLstnSc; sock = sock->next){
 		if ((sock->srcport == tcp->dstport) && ((sock->srcip == INADDR_ANY) || (sock->srcip == ip->dstip))){
 			putRecvBufTcp(mbuf, &sock->recvBuf, &sock->lockGate);
-			activeTaskSoonSignal(getWaitTask(sock->listenTask));
+			TaskAwakeSoon(getTaskWait(sock->listenTask));
 			return 1;
 		}
 	}
@@ -324,7 +329,7 @@ int receiveTcp(struct mbuf *mbuf)
  */
 STATIC int waitRemoteData(
 	SOCKET *sock, 
-	const uint i_timeout,
+	const uint i_timeout,		// ॢȥߥ
 	const struct mbuf *i_mbuf)	// Хåե󥯤ƬӤmbuf
 {
 	if (i_timeout == 0) {
@@ -339,7 +344,7 @@ STATIC int waitRemoteData(
 
 			// Ԥ
 			sock->waitTask = getCurrentTask();
-			remainTime = waitTimeSignal(timeout);
+			remainTime = TaskWaitTimer(timeout);
 
 			if (refRecvBuf(&sock->recvBuf) != i_mbuf) {
 				break;
@@ -373,7 +378,7 @@ enum{
 STATIC INLINE uint32_t getSeqNumber()
 {
 	/* 4msǣ䤹RFC793ˡ */
-	return calcMiliSecondFromClock(rdtsc()) * 4;
+	return calcMiliSecondFromClock(getCpuClock()) * 4;
 }
 
 /*
@@ -458,11 +463,9 @@ STATIC INLINE void storeBuf(
 	SOCKET *sock,
 	struct mbuf *mbuf)
 {
-	AggregateList *aggregate = &sock->recvStore;
-
 	ASSERT(mbuf != NULL);
 
-	aggregate->insertEnd(aggregate, &mbuf->list);
+	sock->recvStoreMethod.insertEnd(&sock->recvStore, &mbuf->list);
 }
 
 /*
@@ -472,11 +475,9 @@ STATIC INLINE void storeBackBuf(
 	SOCKET *sock,
 	struct mbuf *mbuf)
 {
-	AggregateList *aggregate = &sock->recvStore;
-
 	ASSERT(mbuf != NULL);
 
-	aggregate->insertHead(aggregate, &mbuf->list);
+	sock->recvStoreMethod.insertHead(&sock->recvStore, &mbuf->list);
 }
 
 /*
@@ -486,9 +487,7 @@ STATIC INLINE void storeBackBuf(
 STATIC INLINE struct mbuf *getStoreBuf(
 	SOCKET *sock)
 {
-	AggregateList *aggregate = &sock->recvStore;
-
-	return aggregate->getHead(aggregate);
+	return sock->recvStoreMethod.getHead(&sock->recvStore);
 }
 
 /*
@@ -498,9 +497,7 @@ STATIC INLINE struct mbuf *getStoreBuf(
 STATIC INLINE int isExistStoreBuf(
 	SOCKET *sock)
 {
-	AggregateList *aggregate = &sock->recvStore;
-	
-	return (aggregate->refHead(aggregate) != NULL) ? YES : NO;
+	return (sock->recvStoreMethod.refHead(&sock->recvStore) != NULL) ? YES : NO;
 }
 
 /*
@@ -509,10 +506,9 @@ STATIC INLINE int isExistStoreBuf(
 void releaseStoreBuf(
 	SOCKET *sock)
 {
-	AggregateList *aggregate = &sock->recvStore;
 	struct mbuf *mbuf;
 
-	while ((mbuf = aggregate->getHead(aggregate)) != NULL) {
+	while ((mbuf = sock->recvStoreMethod.getHead(&sock->recvStore)) != NULL) {
 		m_freem(mbuf);
 	}
 }
@@ -709,7 +705,7 @@ STATIC int makeNewSocket(SOCKET *sock, struct mbuf *mbuf, SOCKET **o_newSock)
 	newSock->dstip			= ip->srcip;
 	newSock->srcip			= sock->srcip;
 	initRecvBuf(&newSock->recvBuf);
-	AggregateListConstructor(&newSock->recvStore);
+	AggregateListConstructor(&newSock->recvStore, &newSock->recvStoreMethod);
 	newSock->waitTask		= sock->waitTask;
 	newSock->seqnum			= getSeqNumber();
 	newSock->acknum			= swapInt32(tcp->seqnum) + 1;
@@ -833,7 +829,8 @@ STATIC void listenConnect(SOCKET *sock)
 /*
  * åɽλ롼
  */
-STATIC void listenRoutin(SOCKET *sock)
+STATIC void listenRoutin(
+	SOCKET *sock)
 {
 	sock->listenTask = getCurrentTask();
 	
@@ -842,7 +839,7 @@ STATIC void listenRoutin(SOCKET *sock)
 
 	for (;;) {
 		if (refRecvBuf(&sock->recvBuf) == NULL) {
-			waitTaskSignal();
+			TaskWait();
 
 			/* å֤ǤʤХåɽλ */
 			if ((sock->stat & SOCK_LISTEN) == 0) {
@@ -853,10 +850,6 @@ STATIC void listenRoutin(SOCKET *sock)
 			listenConnect(sock);
 		}
 	}
-
-	/*  */
-	kfree(sock);
-	sys_exit(0);
 }
 
 /*
@@ -903,30 +896,38 @@ STATIC void releaseListenSocket(SOCKET *sc)
 
 /*
  * å󥹥åɤκ
- * parameters : socket,file discripter
- * return     : 0 or error number
+ *ԳԲġ
+ * return     : error number
  */
-STATIC int makeListenThread(SOCKET *sock, int fd)
+STATIC int makeListenThread(
+	SOCKET *sock, 
+	int fd)
 {
-	/* listenѥץư */
-	switch(forkKernelThread()) {
-	case -1:
-		return -ENOBUFS;
-	case 0:
-		// СåȤǤ褦
-		// եǥץȥȤ򸺤餹
+	int error = forkKernelThread();
+
+	if (error < 0) {
+		return error;
+	}
+	else if (error == 0) {
+		/*
+		 * å󥹥åɳ
+		 */
+		// СåȤǤ褦˥եǥץȥȤ򸺤餹
 		sys_close(fd);
 
 		listenRoutin(sock);
-		// Not reached.
-	}
-	
-	/* listenץưޤԤ */
-	while (sock->next == sock){
-		wait_task();
+
+		// å󥹥åɽλ
+		kfree(sock);
+		sys_exit(0);
 	}
 
-	return 0;
+	// å󥹥åɤưޤԤ
+	while (sock->next == sock) {
+		mili_timer(TASK_TIME);
+	}
+
+	return NOERR;
 }
 
 /*
@@ -936,7 +937,7 @@ STATIC int makeListenThread(SOCKET *sock, int fd)
 STATIC void endListenThread(SOCKET *sock)
 {
 	sock->stat &= ~SOCK_LISTEN;
-	activeTaskSignal(getWaitTask(sock->listenTask));
+	TaskAwake(getTaskWait(sock->listenTask));
 }
 
 /***************************************************************************************************
@@ -1033,7 +1034,7 @@ STATIC INLINE int condition(
  */
 STATIC struct mbuf *getFromRemote(
 	SOCKET *sock,
-	const uint timeout,		// Ԥ֡á
+	const uint timeout,		// Ԥ֡ʥߥá
 	int *o_error)			// error number
 {
 	enum {
@@ -1431,7 +1432,7 @@ STATIC void disconnect(SOCKET *sock)
 STATIC int connectTo(SOCKET *sock)
 {
 	enum {
-		WAIT_CONNECT = 7,			// ³ॢ
+		WAIT_CONNECT = 7000,	// ³ॢȥߥ
 	};
 	struct mbuf *mbuf;
 	IP_HEADER *ip;
@@ -1603,7 +1604,7 @@ STATIC int send(SOCKET *sock, void *msg, const size_t i_size, int flags, struct
 	if ((sock->stat & SOCK_CNCT_ON) == 0){
 		return -ENOTCONN;
 	}
-	
+
 	return sendData(sock, msg, i_size);
 }
 
@@ -1730,20 +1731,20 @@ STATIC int connect(SOCKET *sock)
 
 STATIC int listen(SOCKET *sock,int fd)
 {
-	if (sock->stat & SOCK_LISTEN)
+	if (sock->stat & SOCK_LISTEN) {
 		return -EADDRINUSE;
+	}
 
-	if (searchListenSocket(sock->srcport) != NULL)
+	if (searchListenSocket(sock->srcport) != NULL) {
 		return -EADDRINUSE;
+	}
 
 	sock->next = sock->prev = sock;
 	sock->stat = SOCK_LINK_ON | SOCK_LISTEN;
 	sock->waitTask = getCurrentTask();
 	
-	/* å󥹥åɤγ */
-	makeListenThread(sock, fd);
-	
-	return 0;
+	// å󥹥åɤư
+	return makeListenThread(sock, fd);
 }
 
 STATIC int accept(SOCKET *sock,SOCKET **o_newSock,uint32_t *o_srcIp)
@@ -1764,7 +1765,7 @@ STATIC int accept(SOCKET *sock,SOCKET **o_newSock,uint32_t *o_srcIp)
 			}
 
 			/* Ԥ */
-			waitTaskSignal();
+			TaskWait();
 			newSock = getFromListenSocket(sock);
 			if (newSock != NULL){
 				break;
@@ -1855,7 +1856,7 @@ STATIC int poll(SOCKET *sock, int events)
 		return 0;
 	}
 	if (events & (POLLOUT | POLLWRNORM | POLLWRBAND)) {
-			return POLLOUT | POLLWRNORM | POLLWRBAND;
+		return POLLOUT | POLLWRNORM | POLLWRBAND;
 	}
 
 	return 0;
@@ -1880,8 +1881,8 @@ int initTcp()
 }
 
 /***************************************************************************/
-void test_tcp()
+uint test_tcp()
 {
-	printk("waitNext=%x\n",*(uint*)0x16be64);
+	return seqnumber;
 }
 /****************************************************************************/
