@@ -878,7 +878,7 @@ STATIC int pci_read_ivar(device_t dev, device_t child, int which, u_long *result
 //			*result = pcib_get_hose(dev);
 			break;
 		default:
-			return ENOENT;
+			return -ENOENT;
 	}
 
 	return 0;
@@ -908,7 +908,7 @@ STATIC int pci_write_ivar(device_t dev, device_t child, int which, uintptr_t val
 		case PCI_IVAR_BUS:
 		case PCI_IVAR_SLOT:
 		case PCI_IVAR_FUNCTION:
-			return EINVAL;	/* disallow for now */
+			return -EINVAL;	/* disallow for now */
 		case PCI_IVAR_SECONDARYBUS:
 			cfg->secondarybus = value;
 			break;
@@ -916,7 +916,7 @@ STATIC int pci_write_ivar(device_t dev, device_t child, int which, uintptr_t val
 			cfg->subordinatebus = value;
 			break;
 		default:
-			return ENOENT;
+			return -ENOENT;
 	}
 	return 0;
 }
