@@ -5,14 +5,13 @@
  * Distributed under the terms of the BSD License.
  */
 
-
 #ifndef sys_pthread_h
 #define sys_pthread_h
 
+#include <sys/Thread.h>
 
 #ifndef ASM_FILE
 
-
 enum {
 	PTHREAD_CREATE_JOINABLE = 1,		// default
 	PTHREAD_CREATE_DETACHED,
@@ -32,8 +31,10 @@ struct pthread_attr {
 	int scope;
 };
 
+struct pthreadState {
+	int taskState;
+};
 
-#endif
+#endif // ASM_FILE
 
-
-#endif
+#endif // sys_pthread_h
