@@ -26,14 +26,14 @@ if [ -d $DIR ]; then
 	cd ../
 fi
 
-DIR=test
+DIR=module/console.mod
 if [ -d $DIR ]; then
 	cd $DIR
 	make
-	cd ../
+	cd ../../
 fi
 
-DIR=module/keybord.mod
+DIR=module/ata.mod
 if [ -d $DIR ]; then
 	cd $DIR
 	make
@@ -47,6 +47,13 @@ if [ -d $DIR ]; then
 	cd ../../
 fi
 
+DIR=test
+if [ -d $DIR ]; then
+	cd $DIR
+	make
+	cd ../
+fi
+
 if [ -d copy ]; then
 	cp include/* copy/usr/include/ -r
 	cp ../sys copy/usr/include/ -r
