@@ -33,7 +33,14 @@ if [ -d $DIR ]; then
 	cd ../
 fi
 
-DIR=module/keybord.mod
+DIR=module/console.mod
+if [ -d $DIR ]; then
+	cd $DIR
+	make clean
+	cd ../../
+fi
+
+DIR=module/ata.mod
 if [ -d $DIR ]; then
 	cd $DIR
 	make clean
