@@ -34,9 +34,6 @@ test/tcpservselect
 printf  "\ntest/tcpservpoll\n"
 test/tcpservpoll
 
-printf  "\ntest/test_brk\n"
-test/test_brk
-
 printf  "\ngcc test.c\n"
 gcc test.c
 
@@ -44,15 +41,23 @@ printf  "\na.out\n"
 ./a.out
 rm a.out
 
-printf  "\ndd if=/dev/fd0 of=fd count=200\n"
-dd if=/dev/fd0 of=fd count=200
-rm fd
+#եåԡɥ饤Сܿޤα
+#printf  "\ndd if=/dev/fd0 of=fd count=200\n"
+#dd if=/dev/fd0 of=fd count=200
+#rm fd
 
 printf  "\nftp 172.25.0.4\n"
 ftp 172.25.0.4
+rm test.dat
 
-printf  "\ncp test.dat test.dat2\n"
-cp test.dat test.dat2
+printf  "\ntest/brk\n"
+test/brk
+
+printf  "\ntest/read\n"
+test/read
+
+printf  "\ntest/write\n"
+test/write
 
 printf  "\ntest/pthread_create\n"
 test/pthread_create
@@ -60,12 +65,14 @@ test/pthread_create
 printf  "\ntest/pthread_join\n"
 test/pthread_join
 
+printf  "\ntest/vm\n"
+test/vm
+
 printf  "\ntest/testThread\n"
 test/testThread
 
-printf  "\ntestModule\n"
-test/testModule1 &
-test/testModule2 &
-test/testIntr
-test/deleteModule testModule1
-test/deleteModule testModule2
+printf  "\ntestCache\n"
+test/testCache
+rm test11.dat
+rm test21.dat
+rm test31.dat
