@@ -6,7 +6,7 @@
  */
 
 
-#include<sys/config.h>
+#include <sys/config.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -15,7 +15,7 @@
 char **environ;
 char *__progname = "\0";
 
-extern void initSbrk(const uint);
+extern void initSbrk();
 extern int main(int argc, char **argv, char **envp);
 
 void _start(
@@ -23,9 +23,7 @@ void _start(
 	char *argv[],
 	char **envp)
 {
-	SYS_INFO *sys_info = (SYS_INFO*) SYS_INFO_ADDR;
-
-	initSbrk(sys_info->lastAddr);
+	initSbrk();
 
 	environ = envp;
 
