@@ -17,10 +17,10 @@ static uint lastAddr;
 /*
  * sbrkƥॳν
  */
-void initSbrk(
-	const uint i_lastAddr)
+void initSbrk()
 {
-	lastAddr = i_lastAddr;
+	extern int getLastAddr(uint*);
+	getLastAddr(&lastAddr);
 }
 
 void *sbrk(
