@@ -24,10 +24,12 @@ void readloop(void)
 		n = recvfrom(sockfd, recvbuf, sizeof(recvbuf), 0, pr->sarecv, &len);
 		if (n < 0)
 		{
-			if (errno == EINTR)
+			if (errno == EINTR) {
 				continue;
-			else
+			}
+			else {
 				err_sys("recvfrom error");
+			}
 		}
 		Gettimeofday(&tval, NULL);
 		(*pr->fproc)(recvbuf, n, &tval);
