@@ -1,10 +1,12 @@
 /*
  * pthread_join.c
  *
- * Test pthread_join.
+ *Գסpthread_join()Υƥ
+ *
  */
 
-
+#include <sys/config.h>
+#include <sys/param.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
@@ -12,12 +14,12 @@
 #include <pthread.h>
 #include "lib/syscall_lib.h"
 
-
 static int times;
 
-void do_thread(int *p_times)
+static void do_thread(
+	int *times)
 {
-//	printf("thread %d\n", *p_times);
+//	printf("thread %d\n", *times);
 }
 
 void duplex_join()
@@ -53,7 +55,7 @@ void duplex_join()
 	}
 }
 
-int main(void)
+static void test_pthread_join()
 {
 	pthread_t thread;
 	void *value;
@@ -61,7 +63,7 @@ int main(void)
 	// Arg error 1.
 	if (pthread_create(&thread, NULL, (void*) do_thread, &times) != 0) {
 		printf("pthread_join error, pthread_create error!\n");
-		return 1;
+		exit(1);
 	}
 
 	argError(pthread_join, 0, (uint) &value, 0, 0, 0, 0, "pthread_join 1");
@@ -74,8 +76,13 @@ int main(void)
 
 	// duplex join error.
 	duplex_join();
+}
 
-	printf("pthread_join OK\n");
+int main(void)
+{
+	test_pthread_join();
+
+	printf("System call \"pthread_join\" OK\n");
 
 	return 0;
 }
